/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cmgui_corba_bridge.
 *
 * The Initial Developer of the Original Code is
 * Auckland Uniservices Ltd, Auckland, New Zealand.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
extern "C"
{
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
}
#include "interfaces/CommandData.hh"
#include "interfaces/NodeFieldCreator.hh"
#include "interfaces/Function.hh"
#include "interfaces/TimeSequencePackage.hh"
#include "interfaces/SceneViewer.hh"
#include "interfaces/Region.hh"
#include "CommandDataFactoryImpl.hpp"
#include "NodeFieldCreatorImpl.hpp"
#include "FunctionImpl.hpp"
#include "TimeSequencePackageImpl.hpp"
#include "SceneViewerImpl.hpp"
#include "RegionImpl.hpp"
#include "ComputedFieldImpl.h"
#include <api/cmiss_function.h>
#include <api/cmiss_function_matrix.h>
#include <api/cmiss_time_sequence.h>
#ifndef WIN32
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtk/gtk_cmiss_scene_viewer.h>
#endif
extern "C"
{
#include <api/cmiss_command.h>
#include <command/cmiss.h>
#include <computed_field/computed_field.h>
#include <computed_field/computed_field_window_projection.h>
  //#include <EXTERN.h>
  //#define PERLDLL
  //#include <perl.h>
}

extern int server_closing;

class CommandDataImpl
  : public POA_CMGUIAPI::CommandData,
    public PortableServer::RefCountServantBase
{
public:
  CommandDataImpl(PortableServer::POA_ptr ipoa)
    : poa(ipoa), refcount(1)
  {
#if 0
    char* perl_argv[] = {"perl", NULL};
    CORBA::ULong perl_argc = 1;
#endif

    char* cmgui_argv[] = {strdup("cmgui"), strdup("-console"), NULL};

    /* Work-around for compiler warning... */
#if 0
    if (perl_argv[0] != perl_argv[0])
      ;
    if (perl_argc != perl_argc)
      ;
#endif

    /* PERL_SYS_INIT(&perl_argc, &perl_argv); */
#ifdef WIN32
    impl = create_Cmiss_command_data(2, cmgui_argv, "0.0", NULL, NULL, NULL, 0);
#else
    impl = create_Cmiss_command_data(1, cmgui_argv, "0.0");
    free(cmgui_argv[0]);
    free(cmgui_argv[1]);
#endif
  }

  ~CommandDataImpl()
  {
  }

  void
  defineMemoryBlock(const char* name,
                    const CMGUIAPI::CommandData::OpaqueData& data)
  {
    struct IO_stream_package* package;
    package = Cmiss_command_data_get_IO_stream_package(impl);
    if (package == NULL)
      throw CMGUIAPI::CmguiError();

    /* We own memory_block... */
    const CORBA::Char* memory_block = data.get_buffer();
    CORBA::ULong mb_len = data.length();
    char* memory_block_copy;
    memcpy(memory_block_copy, memory_block, mb_len);

    /* Pass ownership of memory_block to cmgui... */
    if (!IO_stream_package_define_memory_block(package, (char*)name,
                                               memory_block_copy, mb_len))
      throw CMGUIAPI::CmguiError();
  }

  void
  freeMemoryBlock(const char* name)
  {
    /* XXX we can't leak the memory block, so we should free it here
     * somehow...
     */
    struct IO_stream_package* package;
    package = Cmiss_command_data_get_IO_stream_package(impl);
    if (package == NULL)
      throw CMGUIAPI::CmguiError();

    if (!IO_stream_package_free_memory_block(package, (char*)name))
      throw CMGUIAPI::CmguiError();
  }

  CMGUIAPI::NodeFieldCreator_ptr
  createNodeFieldCreator(CORBA::ULong numComponents)
  {
    NodeFieldCreatorImpl* ptr = new NodeFieldCreatorImpl(numComponents, poa);
    if (ptr == NULL)
      throw CMGUIAPI::CmguiError();
    poa->activate_object(ptr);
    return ptr->_this();
  }

  CMGUIAPI::Function_ptr
  createMatrix(CORBA::ULong rows, const CMGUIAPI::CommandData::DoubleSeq& data)
  {
    CORBA::ULong total_size = data.length();
    CORBA::ULong cols = total_size / rows;
    if (total_size != rows * cols)
      throw CMGUIAPI::CmguiError();

    Cmiss_function_id func =
      Cmiss_function_matrix_create
      (
       rows, cols,
       const_cast<CORBA::Double*>(data.get_buffer())
      );
    if (func == NULL)
      throw CMGUIAPI::CmguiError();
   
    FunctionImpl* fi = new FunctionImpl(func, poa);
    
    poa->activate_object(fi);

    return fi->_this();
  }

  CMGUIAPI::TimeSequencePackage_ptr
  createTimeSequencePackage()
  {
    Cmiss_time_sequence_package_id tsp
      = CREATE(Cmiss_time_sequence_package)();

    TimeSequencePackageImpl* ptr =
      new TimeSequencePackageImpl(tsp, poa);
    if (ptr == NULL)
      throw CMGUIAPI::CmguiError();
    poa->activate_object(ptr);
    return ptr->_this();
  }

  void
  executeCommand(const char* command)
  {
    struct Execute_command *ec;
    ec = Cmiss_command_data_get_execute_command(impl);
    if (ec == NULL)
      throw CMGUIAPI::CmguiError();

    if (!Execute_command_execute_string(ec, (char*)command))
      throw CMGUIAPI::CmguiError();
  }

  void
  createSceneViewer(const CORBA::ULong handle,
                    CMGUIAPI::SceneViewerResultMonitor_ptr svrm)
  {
    struct Cmiss_scene_viewer *sv;
    if (CORBA::is_nil(svrm))
    {
      printf("WARNING: createSceneViewer called with nil monitor!\n");
      return;
    }
#ifdef WIN32
    HWND parentHWND = (HWND)handle;
    HWND childHWND = CreateWindow(
                             "cmgui_embed_wndclass",
                             "",
                             WS_CHILD | WS_VISIBLE,
                             0, 0,
                             100, 100,
                             parentHWND,
                             NULL,
                             GetModuleHandle(NULL),
                             NULL
                            );
    if (childHWND == NULL)
    {
      svrm->sceneViewerFailed();
      return;
    }

    sv =
      CREATE(Cmiss_scene_viewer_win32)
      (
       childHWND,
       CMISS_SCENE_VIEWER_DOUBLE_BUFFERING,
       CMISS_SCENE_VIEWER_ANY_STEREO_MODE,
       0, 0, 0
      );

    if (sv == NULL)
    {
      svrm->sceneViewerFailed();
      return;
    }
#else
    GdkNativeWindow xid = (GdkNativeWindow)handle;
    GtkWidget *windowPlug, *sceneViewer, *packBox;

    windowPlug = gtk_plug_new(xid);
    if (windowPlug == NULL)
    {
      svrm->sceneViewerFailed();
      return;
    }

    packBox = gtk_hbox_new(FALSE, 0);
    if (packBox == NULL)
    {
      svrm->sceneViewerFailed();
      return;
    }

    sceneViewer = gtk_cmiss_scene_viewer_new();
    if (sceneViewer == NULL)
    {
      svrm->sceneViewerFailed();
      return;
    }

    gtk_widget_show(sceneViewer);
    gtk_widget_show(packBox);
    gtk_container_add(GTK_CONTAINER(windowPlug), packBox);
    gtk_container_add(GTK_CONTAINER(packBox), sceneViewer);
    gtk_widget_show(windowPlug);

    sv = gtk_cmiss_scene_viewer_get_cmiss_scene_viewer
         (
          GTK_CMISS_SCENE_VIEWER(sceneViewer)
         );
#endif
#ifdef WIN32
    SceneViewerImpl* svi = new SceneViewerImpl(childHWND, sv, poa);
#else
    SceneViewerImpl* svi = new SceneViewerImpl(windowPlug, sv, poa);
#endif
    poa->activate_object(svi);
    svrm->sceneViewerReady(svi->_this());
    svrm->release_ref();
  }

  CMGUIAPI::Region_ptr
  rootRegion()
  {
    Cmiss_region_id root;
    root = Cmiss_command_data_get_root_region(impl);
    if (root == NULL)
      throw CMGUIAPI::CmguiError();

    RegionImpl* ri = new RegionImpl(root, poa);
    poa->activate_object(ri);
    return ri->_this();
  }

  CMGUIAPI::ComputedField_ptr
  findFieldByName(const char* fieldname)
  {
    struct Computed_field* cf;
    struct MANAGER(Computed_field)* cfm;
    cfm = Cmiss_command_data_get_computed_field_manager(impl);
    cf = FIND_BY_IDENTIFIER_IN_MANAGER(Computed_field, name)(const_cast<char*>(fieldname), cfm);
    
    ComputedFieldImpl* cfi = new ComputedFieldImpl(cf, poa);
    poa->activate_object(cfi);
    return cfi->_this();
  }

  CMGUIAPI::ComputedField_ptr
  createWindowProjectionField(const char* name,
                              CMGUIAPI::ComputedField_ptr sourceField,
                              CMGUIAPI::SceneViewer_ptr sceneViewer,
                              unsigned long projType)
  {
    struct Computed_field* cf;
    struct MANAGER(Computed_field)* cfm;

    // We try to convert all the IDs to servants...
    if (CORBA::is_nil(sourceField) || CORBA::is_nil(sceneViewer))
      throw CMGUIAPI::CmguiError();

    PortableServer::ServantBase* sbscfi = poa->reference_to_servant(sourceField);
    PortableServer::ServantBase_var sbscfiv(sbscfi);
    PortableServer::ServantBase* sbsvi = poa->reference_to_servant(sceneViewer);
    PortableServer::ServantBase_var sbsviv(sbsvi);
    
    Cmiss_scene_viewer_id svi = dynamic_cast<SceneViewerImpl*>(sbsvi)->impl;
    struct Computed_field* scfi = dynamic_cast<ComputedFieldImpl*>(sbscfi)->impl;

    // This is not ACCESSd, so don't need to DEACCESS.
    cfm = Cmiss_command_data_get_computed_field_manager(impl);

    // Actually create the computed field...
    // Yet another char* that should have been const char*...
    cf = CREATE(Computed_field)(const_cast<char*>(name));

    // Set it to be a window projection field...
    Computed_field_set_type_window_projection
    (
     cf, scfi, svi, "none_CORBA_sceneviewer",
     0, (Computed_field_window_projection_type)projType, cfm
    );

    // Add it to the manager...
    ADD_OBJECT_TO_MANAGER(Computed_field)(cf, cfm);

    // And return the field to the user...
    ComputedFieldImpl* cfi = new ComputedFieldImpl(cf, poa);
    poa->activate_object(cfi);
    return cfi->_this();
  }

  void add_ref()
  {
    refcount++;
  }

  void release_ref()
  {
    refcount--;
    if (refcount == 0)
    {
      /* DESTROY(Cmiss_command_data)(&impl); */
      PortableServer::ObjectId_var oid = poa->servant_to_id(this);
      poa->deactivate_object(oid);
      server_closing = 1;
    }
  }

  struct Cmiss_command_data*
  GetCMGUICommandData()
  {
    return impl;
  }
private:
  PortableServer::POA_var poa;
  struct Cmiss_command_data* impl;
  CORBA::ULong refcount;
};

CommandDataFactoryImpl::CommandDataFactoryImpl(PortableServer::POA_ptr ipoa)
  : poa(ipoa)
{
#ifndef MAKE_NEW_COMMANDDATAIMPL
  CommandDataImpl* servant = new CommandDataImpl(poa);
  the_command_data = servant->GetCMGUICommandData();
  PortableServer::ObjectId_var oid = poa->activate_object(servant);
  commandDataRef = servant->_this();
  servant->_remove_ref();
#endif
}

struct Cmiss_command_data*
CommandDataFactoryImpl::GetCMGUICommandData()
{
  return the_command_data;
}

CMGUIAPI::CommandData_ptr
CommandDataFactoryImpl::createCommandData()
{
  fflush(stdout);
#ifdef MAKE_NEW_COMMANDDATAIMPL
  POA_CMGUIAPI::CommandData* servant = new CommandDataImpl(poa);
  PortableServer::ObjectId_var oid = poa->activate_object(servant);
  CMGUIAPI::CommandData_ptr objref = servant->_this();
  servant->_remove_ref();
  return objref;
#else
  return commandDataRef;
#endif
}
