/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cmgui_corba_bridge.
 *
 * The Initial Developer of the Original Code is
 * Auckland Uniservices Ltd, Auckland, New Zealand.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "RegionImpl.hpp"
#include "NodeImpl.hpp"
#include "ElementImpl.hpp"
#include "FunctionImpl.hpp"
#include <api/cmiss_function.h>
#include <api/cmiss_function_finite_element.h>

CMGUIAPI::Node_ptr
RegionImpl::createNode(CORBA::Long nodeIdentifier)
{
  Cmiss_node_id node = create_Cmiss_node(nodeIdentifier, impl);
  NodeImpl* ni = new NodeImpl(node, poa);
  poa->activate_object(ni);
  return ni->_this();
}

void
RegionImpl::beginChange()
{
  if (!Cmiss_region_begin_change_API(impl))
    throw CMGUIAPI::CmguiError();
}

void
RegionImpl::endChange()
{
  if (!Cmiss_region_end_change_API(impl))
    throw CMGUIAPI::CmguiError();
}

void
RegionImpl::mergeNode(CMGUIAPI::Node_ptr tnode)
{
  PortableServer::ServantBase* sbnode = poa->reference_to_servant(tnode);
  NodeImpl* ni = dynamic_cast<NodeImpl*>(sbnode);
  if (ni == NULL)
  {
    sbnode->_remove_ref();
    throw CMGUIAPI::CmguiError();
  }
  Cmiss_node_id n = ni->impl, nn;
  if (!(nn = Cmiss_region_merge_Cmiss_node(impl, n)))
  {
    sbnode->_remove_ref();
    throw CMGUIAPI::CmguiError();
  }
  if (nn != n)
  {
    DESTROY(Cmiss_node)(&n);
  }
  sbnode->_remove_ref();
  ni->impl = nn;
}

void
RegionImpl::mergeElement(CMGUIAPI::Element_ptr telement)
{
  PortableServer::ServantBase* sbelement = poa->reference_to_servant(telement);
  ElementImpl* ei = dynamic_cast<ElementImpl*>(sbelement);
  if (ei == NULL)
  {
    sbelement->_remove_ref();
    throw CMGUIAPI::CmguiError();
  }
  Cmiss_element_id e = ei->impl, en;
  if (!(en = Cmiss_region_merge_Cmiss_element(impl, e)))
  {
    sbelement->_remove_ref();
    throw CMGUIAPI::CmguiError();
  }
  if (en != e)
  {
    // We should destroy the old element e(but we can't as there is no API method).
  }
  ei->impl = en;
  sbelement->_remove_ref();
}

CMGUIAPI::Element_ptr
RegionImpl::createElementWithLineShape
(
 CORBA::Long elementIdentifier,
 CORBA::ULong dimension
)
{
  Cmiss_element_id el =
    CREATE(Cmiss_element_with_line_shape)(elementIdentifier, impl, dimension);
  if (el == NULL)
    throw CMGUIAPI::CmguiError();

  ElementImpl* ei = new ElementImpl(el, poa);
  poa->activate_object(ei);

  return ei->_this();
}

CMGUIAPI::Function_ptr
RegionImpl::createStandardInterpolation
(
 const char* fieldName,
 const CMGUIAPI::StringSequence& componentNames
)
{
  Cmiss_function_id f =
    Cmiss_function_finite_element_create_standard_interpolation_rc_constant_time
    (
     impl,
     const_cast<char*>(fieldName),
     componentNames.length(),
     const_cast<char**>(componentNames.get_buffer())
    );
  FunctionImpl* fi = new FunctionImpl(f, poa);
  poa->activate_object(fi);

  return fi->_this();
}
