/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cmgui_corba_bridge.
 *
 * The Initial Developer of the Original Code is
 * Auckland Uniservices Ltd, Auckland, New Zealand.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "CommandDataFactoryImpl.hpp"
#include <iostream>
extern "C" {
#include <command/cmiss.h>
};
#include <api/cmiss_timer.h>
#include <omniORB4/poa.h>
#include "ApplicationBootstrap.hh"
#include <time.h>
#include <stdlib.h>

static void CreatePermanentObjects(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa,
                                   CMGUIAPI::ApplicationBootstrap_ptr bootstrap);

int server_closing = 0;

struct Cmiss_command_data* the_command_data;
CORBA::ORB_var orb;

static unsigned long sleep_time = 500000000;

int
do_corba_work(void *user_data)
{
  bool did_work = false;
  Cmiss_timer_package_id idlepkg = (Cmiss_timer_package_id)user_data;
  while (orb->work_pending())
  {
    did_work = true;
    orb->perform_work();
  }
  if (server_closing)
  {
    orb->shutdown(true);
    exit(0);
  }
  Cmiss_timer_package_add_callback(idlepkg, 0, sleep_time, do_corba_work, idlepkg);
  /* XXX but how else do we do this? */
  if (!did_work)
  {
    sleep_time *= 2;
    if (sleep_time > 20000000)
      sleep_time = 20000000;
  }
  else
  {
    sleep_time /= 2;
    if (sleep_time < 1000000)
      sleep_time = 1000000;
  }
  return 0;
}

int
main(int argc, char** argv)
{
  if (argc < 2)
  {
    printf("Usage: corba_bridge application_bootstrap_ior\n");
    return -1;
  }

  try
  {
    /* Initialise the ORB... */
    char* orb_argv[] =
      {
	"cmgui_corba_bridge",
	"-ORBserverCallTimeOutPeriod", "400",
	"-ORBclientCallTimeOutPeriod", "400",
	"-ORBendPoint", "giop:tcp:127.0.0.1:0",
	NULL
      };
    int orb_argc = sizeof(orb_argv) / sizeof(char*) - 1;
    orb = CORBA::ORB_init(orb_argc, orb_argv);

    CORBA::Object_var appbootstrapObj = orb->string_to_object(argv[1]);
    if (CORBA::is_nil(appbootstrapObj))
    {
      printf("You passed an invalid IOR on the command-line.\n");
      return -1;
    }

    CMGUIAPI::ApplicationBootstrap_var appbootstrap =
      CMGUIAPI::ApplicationBootstrap::_narrow(appbootstrapObj);
    if (CORBA::is_nil(appbootstrap))
    {
      printf("The appbootstrap must implement CMGUIAPI::ApplicationBootstrap(%s doesn't)!\n", argv[1]);
      return -1;
    }

    /* Fetch the root POA... */
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
    
    /* Now make a special MAIN_THREAD POA... */
    CORBA::PolicyList policylist;
    policylist.length(1);
    policylist[0] = poa->create_thread_policy
                    (
                     PortableServer::MAIN_THREAD_MODEL
                    );
    poa = poa->create_POA("MainThreadPOA", poa->the_POAManager(),
                          policylist);

    /* Activate the POA... */
    PortableServer::POAManager_var pman = poa->the_POAManager();
    pman->activate();

    /* Set up all factories... This is also where we notify the cmgui is ready,
     * so from here on, calls will start to happen...
     */
    CreatePermanentObjects(orb, poa, appbootstrap);

    Cmiss_timer_package_id idlepkg
      = Cmiss_command_data_get_timer_package(the_command_data);
    Cmiss_timer_package_add_callback(idlepkg, 0, sleep_time, do_corba_work, idlepkg);

    /* Go to the cmgui main-loop...
     */
    Cmiss_command_data_main_loop(the_command_data);
  }
  catch (const CORBA::SystemException& e)
  {
#undef minor
    std::cerr << "A CORBA exception occurred while starting the server: "
              << e._name()
              << " minor " << (e.minor())
              << std::endl;
  }
  catch (...)
  {
    std::cerr << "An unsupported exception occurred while starting the server."
              << std::endl;
  }
  return 0;
}

void
CreatePermanentObjects(CORBA::ORB_ptr, PortableServer::POA_ptr poa,
                       CMGUIAPI::ApplicationBootstrap_ptr bootstrap)
{
  CommandDataFactoryImpl* cdfi = new CommandDataFactoryImpl(poa);
  the_command_data = cdfi->GetCMGUICommandData();
  PortableServer::ObjectId_var oid = poa->activate_object(cdfi);

  bootstrap->cmguiReady(cdfi->_this());
  bootstrap->release_ref();
  cdfi->_remove_ref();

#ifdef WIN32
  WNDCLASS srvWC;

  srvWC.style = CS_HREDRAW | CS_VREDRAW;
  srvWC.lpfnWndProc = DefWindowProc;
  srvWC.cbClsExtra = 0;
  srvWC.cbWndExtra = 0;
  srvWC.hInstance = GetModuleHandle(NULL);
  srvWC.hIcon = NULL;
  srvWC.hCursor = LoadCursor(NULL, IDC_ARROW);
  srvWC.hbrBackground = (HBRUSH)COLOR_BACKGROUND;
  srvWC.lpszMenuName = NULL;
  srvWC.lpszClassName = "cmgui_embed_wndclass";
  RegisterClass(&srvWC);
#endif
}
