/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _MOZCELLML_H
#define _MOZCELLML_H

#include <mozilla-config.h>
#include <nsISimpleEnumerator.h>
#include <nsCOMPtr.h>
#include <nsIDocument.h>
#include <dom/nsIDOMXMLDocument.h>
#include <nsIDOMElement.h>
#include <nsIDOMNodeList.h>
#include <nsIDOMText.h>
#include <nsInterfaceHashtable.h>
#include <nsIStringBundle.h>
#include <nsArray.h>
#include <nsCOMPtr.h>
#include "IOMG.h"
#include "mozICellMLLocal.h"
#include <nspr.h>
#include <necko/nsIServerSocket.h>
#include <nsIAsyncOutputStream.h>
#include <nsIAsyncInputStream.h>
#include <nsDataHashtable.h>
#include <nsClassHashtable.h>
#include <nsIArray.h>
#include "mozCellMLUnits.h"
#include "mozCellMLLoadError.h"
#ifdef XP_WIN32
#include <nsIRunnable.h>
#include <windows.h>
#endif

#ifdef ENABLE_LEAKTRACE
#define LEAKTRACE_CREATED(x) printf("Created a " #x "\n");
#define LEAKTRACE_DELETED(x) printf("Deleted a " #x "\n");
#else
#define LEAKTRACE_CREATED(x)
#define LEAKTRACE_DELETED(x)
#endif

#define CELLML_NS_1_0 NS_LITERAL_STRING("http://www.cellml.org/cellml/1.0#")
#define CELLML_NS_1_1 NS_LITERAL_STRING("http://www.cellml.org/cellml/1.1#")
#define CMETA_NS_1_0 NS_LITERAL_STRING("http://www.cellml.org/metadata/1.0#")

#define MATHML_NS NS_LITERAL_STRING("http://www.w3.org/1998/Math/MathML")

nsresult FilterNodeListByType(nsIDOMNodeList* nodelist, nsIArray** oarray,
                              PRUint16 keepType);
nsresult FilterNodeListByType(nsIDOMNodeList* nodelist, nsIArray** oarray,
                              PRUint16 keepType, const nsAString& ns);
nsresult FilterNodeListByNameAndTypeNS(nsIDOMNodeList* nodelist, nsIArray** oarray,
                                       PRUint16 keepType,
                                       const nsAString& ns,
                                       const nsAString& name);
nsresult FilterNodeListByNameAndTypeNS2(nsIDOMNodeList* nodelist, nsIArray** oarray,
                                       PRUint16 keepType,
                                       const nsAString& ns1,
                                       const nsAString& ns2,
                                       const nsAString& name);


#ifdef USE_CELLML_API

#define USE_MOZILLA_DOM
// Kludge to prevent CellML_Namespaces.hpp from being included...
#define CELLML_NAMESPACES_HPP
#include "CellML.hpp"

#endif

#ifdef XP_WIN32
class mozCellMLCORBAThread
  : public nsIRunnable
{
public:
  mozCellMLCORBAThread();

  NS_DECL_NSIRUNNABLE;
  NS_DECL_ISUPPORTS;

  /**
   * A function which is called on the main thread(i.e. must not be called
   * from other threads) which causes an nsIRunnable to run on the other
   * thread. Windows messages will be processed on the main thread, and the
   * runnable will run on the CORBA thread. When the run function returns, this
   * function will return control on the main thread.
   */
  void RunOnThread(nsIRunnable* aDoRun);

  /**
   * A function which notifies the thread that it should exit. This must be
   * called only from the main thread. After it has been called, the structure
   * should not be touched, as it could be deleted at any time by the CORBA
   * thread.
   */
  void AutoDestruct();
private:
  ~mozCellMLCORBAThread();
  HANDLE mCORBASendSemaphore, mCORBARecvSemaphore;
  nsIRunnable* mDoRun;
  bool mAutoDestruct;
};
#endif

class mozCellMLVariable;
class mozCellMLVariableResolution;

class mozCellML
  : public mozICellMLLocal
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_MOZICELLML
  NS_DECL_MOZICELLMLLOCAL

  mozCellML();
  mozCellML(nsCAutoString name, PRBool validate);
  void GetImportNumberPrefix(nsIDOMElement* el, PRUint32* asnumber, nsAString& inp);
  mozCellMLLoadError* GetEarlyError();
protected:
  ~mozCellML();

private:
  PRUint32 nextHashNumber;
  PRUint32 functionNumber;
  nsCAutoString title;

#ifdef XP_WIN32
  mozCellMLCORBAThread* corbaThread;
#endif

  typedef struct {
    nsString importPrefix;
    nsString componentName;
  } ComponentAlias;

  nsCOMPtr<nsIDOMElement> docRoot;
  nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable> allVariables;
  nsInterfaceHashtable<nsStringHashKey,mozCellMLVariableResolution> allVariableRes;
  nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable> nonMainVariables;
  nsClassHashtable<nsStringHashKey,ComponentAlias> componentAliases;
  // A unit is addressed as importnumber:component:unit, component can be
  // empty. This table also contains built-in units, addressed directly by
  // their name.
  nsInterfaceHashtable<nsStringHashKey,mozCellMLUnits> allUnits;
  nsDataHashtable<nsISupportsHashKey,PRUint32> importNumbers;
  nsDataHashtable<nsISupportsHashKey,PRBool> loadingImports;
  nsStringArray importURLTable;
  nsCOMArray<mozCellMLUnits> unitsList;

  nsCOMArray<mozCellMLVariable> differentials, simples, constants;
  nsCOMArray<nsIDOMElement> importedRootList;
  nsCOMArray<mozCellMLVariable> order;
  nsCOMArray<mozCellMLVariable> nonMainList;
  nsCOMPtr<nsIMutableArray> allComponents;
  nsCOMPtr<mozITranslationSet> bundle;
  nsCOMPtr<nsIMutableArray> variableList;
  nsCOMArray<mozCellMLVariableResolution> allResolutions;
  nsCString ctrans;

  nsresult FindVariableElements(nsIDOMElement* root);
  nsresult FindVariableElement(const nsAString&, nsIDOMElement*, mozICellMLVariable**);
  nsresult InstallComponent(nsIDOMElement*,
                            nsAString& inpNo, nsAString& componentName);
  nsresult WriteDifferentialAssignment(nsACString& code, PRUint32 i,
                                       mozCellMLVariable* v,
                                       nsCStringArray& functionStack);
  nsresult WriteSimpleEvaluation(nsACString& code, PRUint32 i, mozCellMLVariable* v,
                                 nsCStringArray& functionStack);
  NS_IMETHODIMP ParseCommandLine(const char* data, char** args, PRUint32 maxargs);
  NS_IMETHODIMP FreeCommandLine(char **args);
  PRUint32 SearchDeeplyForEncapsulations(nsIDOMElement*,
                                         nsInterfaceHashtable<nsStringHashKey,
                                                              nsIDOMElement>&);

  void ExtractUnitsFrom(PRUint32, nsIDOMElement*);
  void AddImplicitUnits();
  nsresult SetEarlyError(mozCellMLLoadError*);

  nsresult LinkImportedUnits(PRUint32 impNo, nsIDOMElement*);

  nsresult ExpandEncapsulatedList(nsAString& inp,
                                  nsCOMArray<nsIDOMElement>&,
                                  nsInterfaceHashtable<nsStringHashKey,
                                                       nsIDOMElement>&,
                                  nsInterfaceHashtable<nsStringHashKey,
                                                       nsIDOMElement>&,
                                  nsClassHashtable<nsStringHashKey,nsString>&,
                                  nsIDOMElement*);
  nsresult HashAvailableComponents(nsInterfaceHashtable<nsStringHashKey,
                                                        nsIDOMElement>&,
                                   nsIDOMElement*);
  nsresult AddDirectImports(nsInterfaceHashtable<nsStringHashKey,
                                                 nsIDOMElement>&,
                            nsInterfaceHashtable<nsStringHashKey,
                                                 nsIDOMElement>&,
                            nsIDOMElement*,
                            nsCOMArray<nsIDOMElement>&
                           );
  nsresult AddRelevantConnections(nsIDOMElement*,
                                  nsInterfaceHashtable<nsStringHashKey,
                                                       nsIDOMElement>&,
                                  nsClassHashtable<nsStringHashKey,nsString>&
                                    deeper
                                  );
  void FindImportedComponent(nsAutoString& inp, nsAutoString& comp);
#ifdef USE_CELLML_API
  CellML_Model model;
#endif

  nsCOMArray<nsIDOMElement> allConnections;

  PRBool localValidationError;
  nsString validationErrorMessage;
  nsCOMPtr<nsIMutableArray> allImports;
  void RequestLoadImport(nsIDOMElement* importNode);
  void ImportAll(nsIDOMElement* root);
  void AddComponentAliases(nsIDOMElement*, nsAString&);
  PRBool ComponentIsImported(nsAString& compName, nsAString& impName,
                             nsIDOMElement* root);
  nsCOMPtr<mozCellMLLoadError> early_error;

public:
  nsresult ValidationError(const nsAString& msg);
  mozCellMLVariable* FindVariable(nsAString& inp, nsAString& compName,
                                  nsAString& varName);
  const char* Translate(const char* name);

  nsresult FetchImports(nsIMutableArray** a);
  nsresult ImportLoaded(nsIDOMElement*, nsIDOMElement*);
  PRUint32 GetFunctionNumber();

  PRBool validate;
};

// This is a utility class used by the integrator and fitter...
class WeakServerSocketListenerAdaptor
  : public nsIServerSocketListener
{
public:
  WeakServerSocketListenerAdaptor(nsIServerSocketListener* ss)
    : weakSock(getter_AddRefs(NS_GetWeakReference(ss)))
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSISERVERSOCKETLISTENER
private:
  nsWeakPtr weakSock;
  ~WeakServerSocketListenerAdaptor() {};
};

class WeakInputCallbackAdaptor
  : public nsIInputStreamCallback
{
public:
  WeakInputCallbackAdaptor(nsIInputStreamCallback* ss)
    : weakSock(getter_AddRefs(NS_GetWeakReference(ss)))
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSIINPUTSTREAMCALLBACK
private:
  nsWeakPtr weakSock;
  ~WeakInputCallbackAdaptor() {};
};

class WeakOutputCallbackAdaptor
  : public nsIOutputStreamCallback
{
public:
  WeakOutputCallbackAdaptor(nsIOutputStreamCallback* ss)
    : weakSock(getter_AddRefs(NS_GetWeakReference(ss)))
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSIOUTPUTSTREAMCALLBACK
private:
  nsWeakPtr weakSock;
  ~WeakOutputCallbackAdaptor() {};
};

#endif // _MOZCELLML_H
