/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "mozCellML.h"
#include "IOMG.h"
#include <nsIArray.h>
#include <nsInterfaceHashtable.h>

class mozCellMLVariable
  : public mozICellMLVariable
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_MOZICELLMLVARIABLE

  mozCellMLVariable(nsIDOMElement* var, mozCellML* name);
  void Initialise(nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable>& allVariables,
                  nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable>& nonMainVariables,
                  nsIArray* components);

  /**
   * Precomputes the dependencies on other variables, given a hashtable of
   * other variables. This is really part of the interface, but it uses
   * templates, so it ended up here.
   */
  nsresult PrecomputeDependencies(mozCellML* model,
                                  nsInterfaceHashtable<nsStringHashKey,
                                  mozCellMLVariable>& allVariables,
                                  nsInterfaceHashtable<nsStringHashKey,
                                  mozCellMLVariable>& nonMainVariables);
  nsresult GetVariableName(nsACString& varname);
  nsresult SetVariableName(const nsACString& varname);
  void SetVariableCode(PRInt32 code) { varCode = code; }
  PRInt32 GetCode() { return varCode; }
  nsresult GetSeen(PRBool* seen);
  nsresult SetSeen(PRBool seen);
  nsresult GetDone(PRBool* done);
  nsresult SetDone(PRBool done);
  nsresult GetVariableName(nsAString& name);
  nsresult SetVariableName(const nsAString& name);
  nsresult OrderDependencies(mozCellML*mcm, nsCOMArray<mozCellMLVariable>& ma);
  nsresult GetComputationString(
                                mozCellML* model,
                                nsInterfaceHashtable<nsStringHashKey, mozCellMLVariableResolution>& allVariables,
                                nsDataHashtable<nsISupportsHashKey,
                                                PRUint32>& importNumbers,
                                nsInterfaceHashtable<nsStringHashKey,
                                                     mozCellMLUnits>& allUnits,
                                nsACString& code,
                                nsCStringArray& functionStack);
  void GetLHSResolution(
                        nsInterfaceHashtable<nsStringHashKey, mozCellMLVariableResolution>&
                          allVariables,
                        mozCellMLVariableResolution** ac
                       );

  PRUint32 FetchDegree();
  void SetHigherSpace(PRUint32 xstartAt, PRUint32 dxstartAt);
  void GetUnits(mozCellMLUnits** units);
  nsresult FindUnits(
                     mozCellML* model,
                     nsDataHashtable<nsISupportsHashKey,
                                     PRUint32>& importNumbers,
                     nsInterfaceHashtable<nsStringHashKey,
                                          mozCellMLUnits>& allUnits,
                     nsAutoString& name,
                     mozCellMLUnits** foundUnits
                    );
  nsresult SetupUnits(
                      mozCellML* model,
                      nsDataHashtable<nsISupportsHashKey,
                                      PRUint32>& importNumbers,
                      nsInterfaceHashtable<nsStringHashKey,
                                           mozCellMLUnits>& allUnits
                     );
  void BoundInformSiblings(
                           mozCellML*,
                           nsInterfaceHashtable<nsStringHashKey,mozCellMLVariableResolution>&
                           allRes, double mup, double constv);

  nsresult GetCMetaID(nsAString& id);
private:
  enum EqualsArgumentTypes
  {
    APPLY_DIFF,
    APPLY_MATH,
    CI,
    CN
  };

protected:
  ~mozCellMLVariable();

private:
  nsresult ClassifyEqualsArgument(nsIDOMNode* node, EqualsArgumentTypes* type,
                                  PRUint32* xdegree,
                                  mozCellML* mcm);
  nsresult DetermineType(nsInterfaceHashtable<nsStringHashKey,
                         mozCellMLVariable>& allVariables,
                         nsInterfaceHashtable<nsStringHashKey,
                         mozCellMLVariable>& nonMainVariables,
                         nsIArray* components,
                         mozCellML*);
  nsresult GetDegree(nsIDOMElement*, PRUint32*);
  // This is autostring to save an extra copy...
  nsresult GetLHSVariableName(mozCellML* model,
                              nsIDOMNode* left, nsAutoString& leftName,
                              PRBool allowDiff = PR_TRUE);
  nsresult GetBoundVariableName(mozCellML* mcm,
                                nsIDOMNode* left, nsAutoString& leftName);
  nsresult SearchForDependencies(
                                 mozCellML* model,
                                 nsInterfaceHashtable<nsStringHashKey,
                                 mozCellMLVariable>& allVariables,
                                 nsInterfaceHashtable<nsStringHashKey,
                                 mozCellMLVariable>& nonMainVariables,
                                 nsAString& comp,
                                 nsIDOMElement* el);
  nsresult GetComputationString(
                                mozCellML* model,
                                nsIDOMElement* from,
                                nsInterfaceHashtable<nsStringHashKey, mozCellMLVariableResolution>& allVariables,
                                nsDataHashtable<nsISupportsHashKey,
                                                PRUint32>& importNumbers,
                                nsInterfaceHashtable<nsStringHashKey,
                                                     mozCellMLUnits>& allUnits,
                                nsACString& code,
                                nsCStringArray& functionStack);
  nsresult GetCNValue(nsIDOMNode* cn, double* value);

  /* Think of this as private, this is a MSVC bug workaround...*/
public:
  PRUint32 type;
  nsCOMPtr<nsIDOMElement> variable;
private:
  PRBool seen, done, AlreadyInIVCall;
  PRUint32 higher_space_dx, higher_space_x;
  PRUint32 degree;
  PRUint32 impNo;
  nsCOMPtr<nsIDOMElement> assignTo, assignFrom;
  nsString assignComponent, assignName;
  nsCOMPtr<nsIMutableArray> dependencies;
  mozCellML *mozcellml;
  nsCString varname;
  PRInt32 varCode;

  nsCOMPtr<mozCellMLUnits> units;
};

enum VariablePromiseResult
{
  PROMISE_SATISFIED,
  PROMISE_LOOP,
  PROMISE_BROKEN
};

class mozCellMLVariablePromise
  : public nsISupports
{
private:
  nsStringArray parties;
  nsStringArray partiesFrom;

protected:
  ~mozCellMLVariablePromise() {}

public:
  nsCOMPtr<mozCellMLVariablePromise> redundantBy;
  nsresult PartySolved(
                       mozCellML* model,
                       nsAString& party,
                       mozCellMLVariableResolution* var,
                       nsInterfaceHashtable<nsStringHashKey,
                                            mozCellMLVariable>& nmt,
                       nsInterfaceHashtable<nsStringHashKey,
                                            mozCellMLVariableResolution>& rt,
                       nsCOMArray<mozCellMLVariableResolution>& allRes);
public:
  NS_DECL_ISUPPORTS

  mozCellMLVariablePromise(nsAString&, nsAString&);
  void AddParty(nsAString&, nsAString&);
  nsStringArray& SetRedundant(mozCellMLVariablePromise* r,
                              nsStringArray** ret2);
  void MergeIn(mozCellMLVariablePromise* merge);

  nsresult Resolve(
                   mozCellML* model,
                   nsInterfaceHashtable<nsStringHashKey,
                                        mozCellMLVariable>& nmt,
                   nsInterfaceHashtable<nsStringHashKey,
                                        mozCellMLVariableResolution>& restable,
                   nsInterfaceHashtable<nsStringHashKey,
                                        mozCellMLVariable>& table,
                   nsCOMArray<mozCellMLVariableResolution>& allRes
                  );
};
