/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 */
#include "mozCellML.h"
#include "mozCellMLProtocol.h"
#include "mozCellMLModule.h"
#include "mozCellMLAboutRedirector.h"
#include "mozCellMLCSVParser.h"
#include "TranslationSet.h"
#include <nsIGenericFactory.h>

#ifdef USE_RUNONCE
#include "mozCellMLRunOnce.h"
#include <nsICategoryManager.h>
#include <nsIServiceManagerUtils.h>
#endif

NS_GENERIC_FACTORY_CONSTRUCTOR(mozCellML)
NS_GENERIC_FACTORY_CONSTRUCTOR(mozCellMLCSVParser)
NS_GENERIC_FACTORY_CONSTRUCTOR(TranslationSet)

static NS_IMETHODIMP
mozCellMLProtocolConstructor(nsISupports* aOuter, REFNSIID aIID,  void** aResult)
{
  nsresult rv;
  mozCellMLProtocol* proto;

  *aResult = nsnull;
  
  if (aOuter != nsnull)
  {
    rv = NS_ERROR_NO_AGGREGATION;
    return rv;
  }

  proto = new mozCellMLProtocol(false);
  
  if (proto == nsnull)
  {
    rv = NS_ERROR_OUT_OF_MEMORY;
    return rv;
  }

  NS_ADDREF(proto);
  rv = proto->QueryInterface(aIID, aResult);
  NS_RELEASE(proto);

  return rv;
}

static NS_IMETHODIMP
mozCellMLValidatingProtocolConstructor(nsISupports* aOuter, REFNSIID aIID,  void** aResult)
{
  nsresult rv;
  mozCellMLProtocol* proto;
  
  *aResult = nsnull;
  
  if (aOuter != nsnull)
  {
    rv = NS_ERROR_NO_AGGREGATION;
    return rv;
  }

  proto = new mozCellMLProtocol(true);
  
  if (proto == nsnull)
  {
    rv = NS_ERROR_OUT_OF_MEMORY;
    return rv;
  }

  NS_ADDREF(proto);
  rv = proto->QueryInterface(aIID, aResult);
  NS_RELEASE(proto);
  
  return rv;
}

#ifdef USE_RUNONCE
static NS_METHOD
mozCellMLRegistration(
                      nsIComponentManager *aCompMgr,
                      nsIFile *aPath,
                      const char *registryLocation,
                      const char *componentType,
                      const nsModuleComponentInfo *info
                     )
{
  nsCOMPtr<nsICategoryManager> catman = do_GetService(NS_CATEGORYMANAGER_CONTRACTID);

  char* previous;
  catman->AddCategoryEntry("app-startup",
                           "mozCellMLRunOnce",
                           "@physiome.org/mozCellMLRunOnce;1",
                           PR_TRUE,
                           PR_TRUE,
                           &previous);
  return NS_OK;
}

static NS_METHOD
mozCellMLDeregistration(
                        nsIComponentManager *aCompMgr,
                        nsIFile *aPath,
                        const char *registryLocation,
                        const nsModuleComponentInfo *info
                       )
{
  return NS_OK;
}
#endif

static const nsModuleComponentInfo components[] = 
{ 
  {
    "mozCellML",
    MOZCELLML_CID,
    "@physiome.org/mozCellML;1",
    mozCellMLConstructor
#ifdef USE_RUNONCE
    ,mozCellMLRegistration,
    mozCellMLDeregistration
#endif
  },
  {
    "mozCellMLProtocol",
    MOZCELLML_PROTOCOL_CID,
    "@mozilla.org/network/protocol;1?name=mozcellml",
    mozCellMLProtocolConstructor
  },
  {
    "mozCellMLProtocol",
    MOZCELLML_PROTOCOL_VALIDATING_CID,
    "@mozilla.org/network/protocol;1?name=mozcellml-validate",
    mozCellMLValidatingProtocolConstructor
  },
  {
    "mozCellMLAboutMozCellML",
    MOZCELLML_ABOUT_REDIRECTOR_MODULE_CID,
    NS_ABOUT_MODULE_CONTRACTID_PREFIX "mozcellml",
    mozCellMLAboutRedirector::Create 
  },
#ifdef USE_RUNONCE
  {
    "mozCellMLRunOnce",
    MOZCELLML_RUNONCE_CID,
    "@physiome.org/mozCellMLRunOnce;1",
    mozCellMLRunOnceConstructor
  },
#endif
  {
    "mozCellMLCSVParser",
    MOZCELLML_CSVPARSER_CID,
    "@physiome.org/mozCellMLCSVParser;1",
    mozCellMLCSVParserConstructor
  },
  {
    "TranslationSet",
    MOZCELLML_TRANSLATIONSET_CID,
    "@physiome.org/TranslationSet;1",
    TranslationSetConstructor
  }
};

NS_IMPL_NSGETMODULE(mozCellMLModule,components)
