/**
 * The VWindow override function for general windows.
 * @param vwin The VWindow to override.
 */
function GeneralWindowOverride(vwin)
{
  vwin.originalPerformClose = vwin.performClose;
  vwin.performClose = CloseGeneralWindow;
  
  if (window.last_internid == null)
    window.last_internid = 0;
  vwin.id = window.last_internid++;
  
  if (window.general_windows == null)
    window.general_windows = [];
  
  vwin.general_windows = window.general_windows;
  window.general_windows[vwin.id] = vwin;
  vwin.updateModel = UpdateModel;
  
  vwin.restoreContent = GeneralWindowRestoreContent;
  vwin.saveContent = GeneralWindowSaveContent;
  vwin.setupForContent = GeneralWindowSetupContent;
  
  vwin.variables_UpdateModel = VariablesUpdateModel;
  vwin.variables_Init = InitVariables;
  vwin.variables_Finish = FinishVariables;
  vwin.variables_SaveContent = VariablesSaveContent;
  vwin.variables_RestoreContent = VariablesRestoreContent;

  vwin.integrationparams_Init = InitIntParams;
  vwin.integrationparams_UpdateModel = IntParamsUpdateModel;
  vwin.integrationparams_Finish = FinishIntParams;
  vwin.integrationparams_SaveContent = IntParamsSaveContent;
  vwin.integrationparams_RestoreContent = IntParamsRestoreContent;

  vwin.integrationresultgraphs_Init = InitGraphs;
  vwin.integrationresultgraphs_UpdateModel = GraphsUpdateModel;
  vwin.integrationresultgraphs_Finish = GraphsFinish;
  vwin.integrationresultgraphs_SaveContent = GraphsSaveContent;
  vwin.integrationresultgraphs_RestoreContent = GraphsRestoreContent;

  vwin.editgraph_Init = InitGraphEditor;
  vwin.editgraph_UpdateModel = GraphEditorUpdateModel;
  vwin.editgraph_Finish = GraphEditorFinish;
  vwin.editgraph_SaveContent = GraphEditorSaveContent;
  vwin.editgraph_RestoreContent = GraphEditorRestoreContent;

  vwin.rawintegrationresults_Init = InitRawResults;
  vwin.rawintegrationresults_UpdateModel = RawResultsUpdateModel;
  vwin.rawintegrationresults_Finish = RawResultsFinish;
  vwin.rawintegrationresults_SaveContent = RawResultsSaveContent;
  vwin.rawintegrationresults_RestoreContent = RawResultsRestoreContent;
}

/**
 * The callback for when the VWindow is been closed.
 */
function CloseGeneralWindow()
{
  this.general_windows[this.id] = null;
  this.originalPerformClose();
}

/**
 * The handler for the "View" select-box being changed.
 * @param newview The ID of the new view to show.
 * @param box The VWindow box.
 */
function ChangeGeneralView(newview, box, x)
{
  if (x != null)
  {
    // This is an ugly workaround for a Mozilla bug, but it works(I hope).
    var gp = x.parentNode.parentNode;
    gp.removeChild(x.parentNode);
    var n = x.parentNode.cloneNode(true);
    gp.appendChild(n);
  }
  var contentbox = box.getElementsByTagName("vbox").item(0);
  var vw = box.parentNode.parentNode.vwindow;

  if (vw.currentView != null)
  {
    var fn = vw.currentView + "_Finish";
    if (vw[fn] != null)
      vw[fn](true);
  }

  vw.currentView = newview;

  box.removeChild(contentbox);
  contentbox = box.ownerDocument.importNode
               (
                 vw.repoDocument.getElementById(newview), true
               );
  box.appendChild(contentbox);

  var fn = newview + "_Init";
  if (vw[fn] != null)
    vw[fn](contentbox);

  if (vw.general_current_model != null)
  {
    var fn = newview + "_UpdateModel";
    if (vw[fn] != null)
      vw[fn](vw.general_current_model);
  }
}

/**
 * The callback for when the global model is changed on the sidebar.
 * @param m The newly chosen ModelInstance.
 */
function UpdateModel(m)
{
  // If we are locked, completely ignore the UpdateModel request...
  if (this.lockon.getAttribute("checked") == "true")
  {
    return;
  }
  var funcn = this.currentView + "_UpdateModel";
  this.general_current_model = m;
  if (this[funcn] != null)
    this[funcn](m);
}

/**
 * The callback for when the window content is being restored from XML.
 * @param node The element node from which the window content is being restored.
 */
function GeneralWindowRestoreContent(node)
{
  this.hideUnimportant = node.hasAttribute("hideUnimportant");
  this.drawTree = node.hasAttribute("drawTree");

  var vt = node.getAttribute("viewtype");
  this.view_select.setAttribute("value", vt);
  var vsce = this.view_select.QueryInterface(Components.interfaces.nsIDOMXULSelectControlElement);
  vsce.value = vt;
  ChangeGeneralView(vt,
                    this.contentArea.getElementsByTagName("vbox").item(0), null);
  this.lockon.setAttribute("checked", "false");
  var activeModelID = node.getAttribute("activeModel");
  if (activeModelID != -1 && window.model_table[activeModelID] != null)
  {
    this.updateModel(window.model_table[activeModelID]);
  }
  this.lockon.setAttribute("checked", node.getAttribute("lockmodel"));

  var funcn = this.currentView + "_RestoreContent";
  if (this[funcn] != null)
    this[funcn](node);
}

/**
 * The callback for when the window content is being serialised to XML.
 * @param node The element node onto which window content is being saved.
 */
function GeneralWindowSaveContent(node)
{
  node.setAttribute("lockmodel", this.lockon.getAttribute("checked"));
  node.setAttribute("viewtype", this.view_select.getAttribute("value"));
  if (this.activeModel == null)
    node.setAttribute("activeModel", "-1");
  else
    node.setAttribute("activeModel", this.activeModel.instance_id);

  // Force any state saved only in UI onto the VWindow.
  var fn = this.currentView + "_Finish";
  if (this[fn] != null)
    this[fn](false);

  // We now save state left over from each window type...
  if (this.hideUnimportant)
    node.setAttribute("hideUnimportant", this.hideUnimportant);
  if (this.drawTree)
    node.setAttribute("drawTree", this.drawTree);

  var funcn = this.currentView + "_SaveContent";
  if (this[funcn] != null)
    this[funcn](node);
}

/**
 * The callback for when the window content is first added.
 * @param node The content box node.
 */
function GeneralWindowSetupContent(node)
{
  this.lockon = node.getElementsByTagName("checkbox").item(0);
  this.view_select = node.getElementsByTagName("menulist").item(0);
}
