function InitGraphs(box)
{
  this.graphouter = box;
  this.graphouter.vwin = this;
  var hb = box.getElementsByTagName("hbox").item(0);
  this.menulist = hb.getElementsByTagName("menulist").item(0);
  this.menupopup = this.menulist.getElementsByTagName("menupopup").item(0);
  this.activeModel = null;
  this.activeGraph = null;
}

function GraphsConsiderRescale(m, w)
{
  this.m = m;
  this.w = w;
  return this;
}

function RemoveGraphMonitor(g, m)
{
  var x;
  for (x in g.graph_monitors)
  {
    if (g.graph_monitors[x] == m)
      delete g.graph_monitors[x];
  }
}

GraphsConsiderRescale.prototype =
{
  scaleChanged: function(v, ns)
  {
    var isScaleDirty = false;
    // Look for traces involving this variable...
    if (this.w.activeGraph != null)
    {
      if (ns < 0)
        ns = -ns;
      for (tid in this.w.activeGraph.traces)
      {
        var t = this.w.activeGraph.traces[tid];
        if (t.xvarlev.cellmlvar == v && this.w.xscale < ns)
        {
          isScaleDirty = true;
          this.w.xscale = ns;
        }
        if (t.yvarlev.cellmlvar == v && this.w.yscale < ns)
        {
          isScaleDirty = true;
          this.w.yscale = ns;
        }
      }
      if (isScaleDirty)
      {
        RedefineGraphScale(this.w);
      }
    }
  },

  progress: function(pd)
  {
  },
  
  done: function()
  {
  },

  halted: function()
  {
  },

  failed: function(msg)
  {
  }
};

function SaveGraphViewState(sv, g)
{
  var l = {}, r = {}, b = {}, t = {}, n = {}, f = {};
  if (sv == null || g == null)
    return;
  sv.getViewingVolume(l, r, b, t, n, f);
  g.left = l.value;
  g.right = r.value;
  g.bottom = b.value;
  g.top = t.value;

  var ex = {}, ey = {}, ez = {}, lx = {}, ly = {}, lz = {};
  var ux = {}, uy = {}, uz = {};
  sv.getLookatParameters(ex, ey, ez, lx, ly, lz, ux, uy, uz);
  g.eyeX = ex.value;
  g.eyeY = ey.value;
  g.eyeZ = ez.value;
  g.lookatX = lx.value;
  g.lookatY = ly.value;
  g.lookatZ = lz.value;
}

function RestoreGraphViewState(sv, g)
{
  if (sv == null || g == null)
    return;
  if (g.left == null)
  {
    g.left = -1.0;
    g.right = 1.0;
    g.bottom = -1.0;
    g.top = 1.0;
    g.eyeX = 0.0;
    g.eyeY = 0.0;
    g.eyeZ = 2.0;
    g.lookatX = 0.0;
    g.lookatY = 0.0;
    g.lookatZ = 0.0;
  }
  sv.setViewingVolume(g.left, g.right, g.bottom, g.top, 0.1, 1000);
  sv.setLookatParameters(g.eyeX, g.eyeY, g.eyeZ, g.lookatX, g.lookatY,
                         g.lookatZ, 0.0, 1.0, 0.0);
}

function GraphsUpdateModel(model)
{
  if (this.activeModel == model)
    return;

  this.integrationresultgraphs_Finish(true);

  this.activeModel = model;

  // If this model doesn't have any graphs defined, bounce the user across to
  // the graph editor...
  var g1 = null;
  if (model.xmlmi.graphs != null)
  {
    var i;
    for (i in model.xmlmi.graphs)
    {
      g1 = model.xmlmi.graphs[i];
      break;
    }
  }
  if (g1 == null)
  {
    var gop = this.graphouter.parentNode;
    if (gop == null)
    {
      alert("Null graph outer parent.");
      return;
    }
    var ml = gop.getElementsByTagName("menulist").item(0);
    ml.QueryInterface(Components.interfaces.nsIDOMXULSelectControlElement).value = "editgraph";
    ChangeGeneralView("editgraph", gop, null);
    return;
  }
  if (this.activeGraph != null)
  {
    SaveGraphViewState(this.sceneViewer, this.activeGraph);
    RemoveGraphMonitor(this.activeGraph, this);
  }
  // The active graph is the first graph, if there is no other choice...
  this.activeGraph = model.activeGraph;
  if (this.activeGraph == null)
  {
    this.activeGraph = g1;
    model.activeGraph = this.activeGraph;
  }

  if (this.activeGraph.graph_monitors == null)
    this.activeGraph.graph_monitors = [];
  this.activeGraph.graph_monitors.push(this);

  this.newGraphTypeCreated = GraphViewerNewGraphType;
  this.graphTypeDeleted = GraphViewerDeleteGraphType;
  this.traceDeleted = GraphViewerTraceDeleted;
  this.traceVariablesChanged = GraphViewerTraceVariablesChanged;
  this.traceCreated = GraphViewerTraceCreated;
  this.traceColourChanged = GraphViewerTraceColourChanged;
  if (this.activeGraph.graph_monitors == null)
    this.activeGraph.graph_monitors = [];
  this.activeGraph.graph_monitors.push(this);
  if (model.xmlmi.graph_list_monitors == null)
    model.xmlmi.graph_list_monitors = [];
  model.xmlmi.graph_list_monitors.push(this);

  var i;
  var setOne = null;

  // Wipe out the menupopup...
  var mil = this.menupopup.getElementsByTagName("menuitem");
  for (i = 0; i < mil.length; i++)
  {
    this.menupopup.removeChild(mil.item(i));
  }

  if (this.activeModel.selectedGraph != null)
    setOne = model.xmlmi.selectedGraph;

  // Populate the menupopup...
  for (i in model.xmlmi.graphs)
  {
    var mi = this.win.document.createElementNS
      (
       "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
       "menuitem"
      );
    mi.setAttribute("label", model.xmlmi.graphs[i].name);
    mi.setAttribute("value", model.xmlmi.graphs[i].id);
    if (setOne == null || setOne == i)
    {
      setOne = i;
      mi.setAttribute("selected", "true");
    }
    this.menupopup.appendChild(mi);
  }
  this.menulist.QueryInterface(Components.interfaces
                               .nsIDOMXULSelectControlElement
                              ).value = setOne;

  // If integration hasn't started yet, bounce the user to the
  //  integration params page...
  if (this.activeModel.gsm == null)
  {
    var gop = this.graphouter.parentNode;
    var ml = gop.getElementsByTagName("menulist").item(0);
    ml.QueryInterface(Components.interfaces.nsIDOMXULSelectControlElement)
      .value = "integrationparams";
    ChangeGeneralView("integrationparams", gop, null);
    return;
  }

  this.monitor = new GraphsConsiderRescale(model, this);
  this.activeModel.gsm.addMonitor(this.monitor);

  try
  {
    if (this.currentEmbed != null)
      this.graphouter.removeChild(this.currentEmbed);
  }
  catch (e) {}
  
  if (model.cmgui == null)
  {
    return;
  }

  this.currentEmbed = this.doc.createElementNS("http://www.w3.org/1999/xhtml",
                                               "embed");
  this.currentEmbed.setAttribute("type", "application/x-general-plugin");
  this.currentEmbed.setAttribute("flex", "1");
  this.graphouter.appendChild(this.currentEmbed);
  this.activeModel = model;

  var x = this;
  var tof = function()
  {
    try
    {
      x.plugin = x.currentEmbed
                  .QueryInterface(Components.interfaces.mozIGeneralPlugin);
      GraphPluginReady(x);
    }
    catch (e)
    {
      setTimeout(tof, 0);
    }
  }
  setTimeout(tof, 0);
}

function GraphsFinish(destroy)
{
  if (!destroy)
    return;
  if (this.activeModel != null)
  {
    if (this.activeModel.gsm != null)
      this.activeModel.gsm.removeMonitor(this.monitor);
    var x;
    for (x in this.activeModel.xmlmi.graph_list_monitors)
    {
      if (this.activeModel.xmlmi.graph_list_monitors[x] == this)
        delete this.activeModel.xmlmi.graph_list_monitors[x];
    }
    if (this.activeGraph != null)
    {
      SaveGraphViewState(this.sceneViewer, this.activeGraph);
      RemoveGraphMonitor(this.activeGraph, this);
    }
  }
  if (this.sceneViewer != null)
  {
    this.sceneViewer.setSceneByName("default");

    // Destroy the scene...
    DestroyScene(this);
  }

  this.plugin = null;
  this.sceneViewer = null;
}

function GraphPluginReady(w)
{
  var pluginWindow = w.plugin.window;
  var width = w.plugin.width;
  var height = w.plugin.height;
  PrepareNewScene(w, w.activeModel, pluginWindow, width, height);
}

function ComputeTraceScale(monitor, varlev)
{
  // Look for traces involving this variable...
  var gtsa = this.w.graphTraceScales;
  if (ns < 0)
    ns = -ns;
  for (tid in this.w.graphTraceScales)
  {
    if (gtsa[tid].xvar.cellmlvar == v && gtsa[tid].xscale < ns)
      gtsa[tid].xscale = ns;
    if (gtsa[tid].yvar.cellmlvar == v && gtsa[tid].yscale < ns)
      gtsa[tid].yscale = ns;
  }
}

function SelectDrawGraph(vwin, value)
{
  if (vwin.notUser != 0)
    return;
  // Destroy the old graph...
  if (vwin.activeGraph != null)
  {
    DestroyGraph(vwin, vwin.activeGraph);
    SaveGraphViewState(vwin.sceneViewer, vwin.activeGraph);
    RemoveGraphMonitor(vwin.activeGraph, vwin);
  }
  // Set the active graph...
  vwin.activeGraph = vwin.activeModel.xmlmi.graphs[value];
  vwin.activeModel.activeGraph = vwin.activeGraph;
  vwin.activeGraph.graph_monitors.push(vwin);
  if (vwin.sceneViewer != null)
    vwin.sceneViewer.drawGraph(vwin.activeGraph);

  vwin.activeModel.selectedGraph = value;
}

function GraphViewerNewGraphType(name, id)
{
  var mi = this.win.document.createElementNS
    (
     "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
     "menuitem"
    );
  var m = this.activeModel;
  mi.setAttribute("label", m.xmlmi.graphs[id].name);
  mi.setAttribute("value", m.xmlmi.graphs[id].id);
  this.menupopup.appendChild(mi);
}

function GraphViewerDeleteGraphType(graph)
{
  var l = this.menupopup.getElementsByTagName("menuitem");
  var i;
  for (i = 0; i < l.length; i++)
  {
    var ii = l.item(i);
    if (ii.getAttribute("value") != graph.id)
      continue;
    this.menupopup.removeChild(ii);
    break;
  }

  // If the deleted graph was inactive, we are done...
  if (this.activeGraph != graph)
    return;

  var replace = null;
  for (i in this.activeModel.xmlmi.graphs)
  {
    replace = this.activeModel.xmlmi.graphs[i];
    break;
  }

  // If the only remaining graph was deleted, bounce to the graph editor so
  // more graphs can be added.
  if (replace == null)
  {
    var gop = this.graphouter.parentNode;
    if (gop == null)
    {
      alert("Null graph outer parent.");
      return;
    }
    var ml = gop.getElementsByTagName("menulist").item(0);
    ml.QueryInterface(Components.interfaces.nsIDOMXULSelectControlElement).value = "editgraph";
    ChangeGeneralView("editgraph", gop, null);
    return;
  }

  // Destroy the scene data...
  DestroyGraph(this, graph);

  if (this.activeGraph != null)
  {
    SaveGraphViewState(this.sceneViewer, this.activeGraph);
    RemoveGraphMonitor(this.activeGraph, this);
  }
  this.activeGraph = replace;
  this.activeGraph.graph_monitors.push(this);

  // Draw the new graph...
  this.sceneViewer.drawGraph(this.activeGraph);

  // Update the select box...
  this.notUser++;
  this.menulist.QueryInterface(Components.interfaces.
                               nsIDOMXULSelectControlElement).value = this.activeGraph.id;
  this.notUser--;
}

function GraphViewerTraceDeleted(graph, whichTrace)
{
  // We have to destroy the entire graph and make a new one, as numbering
  // changed...
  DestroySingleTrace(this, this.activeGraph.last_trace_id);
  DestroyGraph(this, this.activeGraph);
  this.sceneViewer.drawGraph(this.activeGraph);
}

function GraphViewerTraceCreated(graph, traceId, xname, yname)
{
  // We destroy the entire graph and make a new one...
  DestroyGraph(this, this.activeGraph);
  this.sceneViewer.drawGraph(this.activeGraph);
}

function GraphViewerTraceVariablesChanged(graph, traceId, xname, yname)
{
  // We destroy the entire graph and make a new one...
  DestroyGraph(this, this.activeGraph);
  this.sceneViewer.drawGraph(this.activeGraph);
}

function GraphViewerTraceColourChanged(graph, traceId, trace)
{
  var sn = this.sceneViewer.sceneName;
  var tn = sn + ":trace" + (traceId - 1);
  this.sceneViewer.changeTraceColour(tn, trace.colour);
}

function GraphsSaveContent(node)
{
  // This function is a placeholder for now, as integration results are not
  // saved, and so there is no need to restore graph windows.
}

function GraphsRestoreContent(node)
{
  // This function is a placeholder for now, as integration results are not
  // saved, and so there is no need to restore graph windows.
}
