/**
 * The VWindow override function for load model windows.
 * @param vwin The VWindow to override.
 */
function LoadModelOverride(vwin)
{
  vwin.restoreContent = LoadModelWindowRestoreContent;
  vwin.saveContent = LoadModelWindowSaveContent;
}

/**
 * The callback for when the window content is being restored from XML.
 */
function LoadModelWindowRestoreContent(node)
{
  var tb = this.loadedContent.getElementsByTagNameNS("http://www.w3.org/1999/xhtml","input").item(0);
  tb.value = node.getAttribute("url");
}

/**
 * The callback for when the window content is being serialised to XML.
 */
function LoadModelWindowSaveContent(node)
{
  var tb = this.loadedContent.getElementsByTagNameNS("http://www.w3.org/1999/xhtml","input").item(0);
  node.setAttribute("url", tb.value);
}

function ModelLoadObserver(w)
{
  this.w = w;
  return this;
}

ModelLoadObserver.prototype =
{
  QueryInterface: function (aIID)
  {
    if (
        aIID.equals(Components.interfaces.nsIStreamLoaderObserver) ||
        aIID.equals(Components.interfaces.nsISupports)
       )
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
  },

  onStreamComplete: function (loader, ctxt, status, resultLength, result)
  {
    alert("OSC called.");
    var strb = this.w.document.getElementById("mozcellml-string-bundle");
    var sb = GetStatusBar();
    if (status == 0)
      sb.label = strb.getString("documentLoaded");
    else
    {
      sb.label = strb.getString("loadModelURLFailed");
    }
  }
};

function SelectLocalModel(returnto)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  try
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    fp.init(window, strb.getString("chooseLoadModel"),
            nsIFilePicker.modeOpen);
    fp.appendFilter(strb.getString("allCellML"), "*.cml; *.xml");
    fp.appendFilter("CML", "*.cml");
    fp.appendFilter("XML", "*.xml");
    fp.appendFilter(strb.getString("allFiles"), "*");
    if (fp.show() != nsIFilePicker.returnOK)
      return;
    returnto.value = fp.fileURL.spec;
  }
  catch (ex)
  {
    alert("Exception getting load dialog: " + ex);
  }
}

function UserRequestLoad(location)
{
  var strb = window.top.document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  if (statusBar == null)
    statusBar = GetStatusBar();
  statusBar.setAttribute("label", strb.getFormattedString("loadingURLLocation", [(""+ location)]));

  location = "mozcellml:" + location;

  var url = Components.classes["@mozilla.org/network/standard-url;1"]
                      .createInstance(Components.interfaces.nsIURI);
  
  var chan = Components.classes["@mozilla.org/network/io-service;1"]
                       .getService(Components.interfaces.nsIIOService)
                       .newChannel(location, null, null);
  url.spec = location;
  chan.originalURI = url;

  window.loader = Components.classes["@mozilla.org/network/stream-loader;1"]
                            .createInstance(Components.interfaces.nsIStreamLoader);
  var mlo = new ModelLoadObserver(window).QueryInterface(Components.interfaces.
                                                         nsIStreamLoaderObserver);
  window.loader.init(chan, mlo, null);
}
