/**
 * Converts the selection and expansion state of a tree into a string.
 * @param t The tree.
 * @return The serialised tree state.
 */
function StringifyTreeState(t)
{
  if (t == null)
    throw Components.results.NS_INVALIDPARAM;
  try
  {
    t = t.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  }
  catch (e)
  {
    alert("Called by: " + e.location.caller.toString() + ": " + e);
    throw e;
  }
  var str = t.view.selection.currentIndex + ",";
  var i;
  for (i = 0; i < t.view.rowCount; i++)
  {
    str += (t.view.isContainerOpen(i) ? "t," : "f,");
  }

  return str;
}

/**
 * Loads a tree state which was previously serialised to a string.
 * @param t The tree.
 * @param state The string containing the serialised tree state.
 */
function LoadTreeState(t, state)
{
  t = t.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  if (state == null)
    return;
  var m = state.split(',');
  var i;
  for (i = 0; i < t.view.rowCount; i++)
    if (t.view.isContainerOpen(i) != (m[i + 1]=="t"))
      t.view.toggleOpenState(i);

  if (m[0] != -1)
  {
    t.view.selection.currentIndex = m[0];
    t.view.selection.toggleSelect(m[0]);
  }
}
