function InitRawResults(content)
{
  content.vwin = this;
  this.tree = content.getElementsByTagName("tree").item(0);
  this.treecols = [];
  var tc = this.tree.getElementsByTagName("treecols").item(0);
  var tcl = tc.getElementsByTagName("treecol");
  
  // We now have to find the data columns...
  for (i = 0; i < tcl.length; i++)
  {
    var col = tcl.item(i);
    var l = col.getAttribute("label");
    if (l == parseInt(l))
      this.treecols.push(col);
  }

  var hb = content.getElementsByTagName("hbox").item(0);
  this.timebox = hb.getElementsByTagName("textbox").item(0).QueryInterface
    (Components.interfaces.nsIDOMXULTextBoxElement);
  this.unitlabel = hb.getElementsByTagName("label").item(0);
  var allIms = hb.getElementsByTagName("image");
  this.imageup = allIms.item(0);
  this.imagedown = allIms.item(1);
  var allCbs = hb.getElementsByTagName("checkbox");
  this.hide_unimportant = allCbs.item(0);
  this.draw_as_tree = allCbs.item(1);
  this.important = allCbs.item(2);
  this.rdfsource_active = null;
  this.rdfindex =
    window.rdfserv.GetResource("http://www.physiome.org/rdf/mozCellML#variableIndex");
  this.tree.window = this;
  this.activeModel = null;
  this.modlev = null;
}

function DoubleToPresentationString(dbl, places)
{
  var pc = "";
  if (dbl < 0)
  {
    dbl = -dbl;
    pc = pc + "-";
  }
  // Get this special case out of the way...
  if (dbl == 0)
    return "0";
  var pcm = Math.pow(10, places);
  if (dbl >= 1 && dbl <= pcm)
  {
    var spareplaces = places - Math.ceil(Math.log(dbl)/Math.log(10));
    var mup = Math.pow(10, spareplaces);
    return "" + (Math.round(dbl * mup) / mup);
  }
  // We need to use XXXEYY notation...
  var exp = Math.floor(Math.log(dbl)/Math.log(10));
  var mup = Math.pow(10, -exp);
  pcm /= 10;
  var pr = (Math.round(dbl * mup * pcm) / pcm);
  // This can happen due to rounding...
  if (pr >= 10)
  {
    pr /= 10;
    exp++;
  }
  var str = "" + pr;
  if (exp != 0)
  {
    str += "E" + exp;
  }
  return str;
}

function ChangeVisibleResultSet(vwin)
{
  if (vwin.activeModel == null)
    return;

  // Find what time we are looking at...
  var whatTime = parseFloat(vwin.timebox.value);

  if (whatTime + 9 * vwin.activeModel.gridx > vwin.activeModel.stopx)
    whatTime = vwin.activeModel.stopx - 9 * vwin.activeModel.gridx;

  // If whatTime doesn't lie on the grid, round down so it does...
  var norm = (whatTime - vwin.activeModel.startx) / vwin.activeModel.gridx;
  var rem = norm - parseInt(norm);
  whatTime -= rem * vwin.activeModel.gridx;

  // If the time is less than start, set it to start...
  if (whatTime < vwin.activeModel.startx)
    whatTime = vwin.activeModel.startx;

  // Fix the column titles...
  var x = whatTime;
  var c;
  for (c = 0; c < 10; c++)
  {
    vwin.treecols[c].setAttribute("label", DoubleToPresentationString(x, 4));
    x += vwin.activeModel.gridx;
  }
  
  // The leftmost entry is guaranteed to be in range, but any to the right of
  // it may not be. So fill in available points and blank the rest...
  var str = "http://www.physiome.org/rdf/mozCellML#point";
  x = whatTime;
  for (c = 1; c <= 10; c++)
  {
    var n = str + c;
    var pred = this.rdfserv.GetResource(n);
    // Loop for each variable...
    var i;
    for (i = 0; i < vwin.activeModel.xmlmi.nivariables; i++)
    {
      var v = vwin.activeModel.xmlmi.ivariables[i];
      var y;
      if (x <= vwin.activeModel.stopx)
      {
        try
        {
          var val = vwin.activeModel.results.
            getValue(i, Math.round((x - vwin.activeModel.startx) /
                                   vwin.activeModel.gridx));
          
          y = DoubleToPresentationString(val, 4);
        }
        catch (e)
        {
          y = "";
        }
      }
      else
        y = "";
      var subj = vwin.vlevs[i].resource;
      var targ = this.rdfserv.GetLiteral(y);
      vwin.rdfsource_everything_tree.Assert(subj, pred, targ, true);
      vwin.rdfsource_everything_flat.Assert(subj, pred, targ, true);
      vwin.rdfsource_important_tree.Assert(subj, pred, targ, true);
      vwin.rdfsource_important_flat.Assert(subj, pred, targ, true);
    }
    x += vwin.activeModel.gridx;
  }
}

function RawResultsUpdateModel(m)
{
  this.rawintegrationresults_Finish(true);
  this.modlev = null;

  this.activeModel = m;
  // If integration hasn't started yet, bounce the user to the
  //  integration params page...
  if (this.activeModel.gsm == null)
  {
    var gop = this.tree.parentNode.parentNode;
    var ml = gop.getElementsByTagName("menulist").item(0);
    ml.QueryInterface(Components.interfaces.nsIDOMXULSelectControlElement)
      .value = "integrationparams";
    ChangeGeneralView("integrationparams", gop, null);
    return;
  }

  this.timebox.value = this.activeModel.startx;
  this.unitlabel.setAttribute("value", this.activeModel.xmlmi.bvar.units);

  this.rdfserv = window.rdfserv;
  this.rdfcu = window.rdfcu;

  this.rdfsource_everything_tree = 
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);
  this.rdfsource_important_tree = 
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);
  this.rdfsource_everything_flat = 
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);
  this.rdfsource_important_flat = 
    Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
              .createInstance(Components.interfaces.nsIRDFDataSource);

  this.modlev = new ModelLevel(this.activeModel, this.rdfserv,
                               this.rdfsource_everything_tree,
                               this.rdfsource_important_tree,
                               this.rdfsource_everything_flat,
                               this.rdfsource_important_flat,
                               this.rdfcu);
  // Make a new variable level for every variable in ivariables...
  var i;
  this.vlevs = [];
  for (i = 0; i < this.activeModel.xmlmi.nivariables; i++)
  {
    var v = this.activeModel.xmlmi.ivariables[i];
    this.vlevs[i] = new VariableLevel(this.modlev, v.name, v, i);
  }

  if (this.activeModel.xmlmi.hideUnimportant)
    this.hide_unimportant.setAttribute("checked", "true");
  else
    this.hide_unimportant.removeAttribute("checked");

  if (this.activeModel.xmlmi.drawTree)
    this.draw_as_tree.setAttribute("checked", "true");
  else
    this.draw_as_tree.removeAttribute("checked");
  SelectRawDataTreeLayout(this);

  // Set the visible result set...
  if (this.activeModel.xmlmi.resultTime != null)
    this.timebox.value = DoubleToPresentationString(this.activeModel.xmlmi.resultTime, 5);

  if (this.activeModel.xmlmi.lastImportantState != null)
  {
    this.modlev.restoreImportantState(this.activeModel.xmlmi.lastImportantState);
  }
  ChangeVisibleResultSet(this);

  if (this.activeModel.xmlmi.lastTreeStateResults != null)
  {
    LoadTreeState(this.tree, this.activeModel.xmlmi.lastTreeStateResults);
  }
}

function RawResultsFinish(destroy)
{
  if (this.activeModel != null)
  {
    this.activeModel.xmlmi.lastTreeStateResults =
      StringifyTreeState(this.tree);
    this.activeModel.xmlmi.resultTime = this.timebox.value;
    if (this.modlev != null)
      this.activeModel.xmlmi.lastImportantState =
        this.modlev.stringifyImportantState();
    if (this.hide_unimportant == null)
      this.activeModel.xmlmi.hideUnimportant = false;
    else
      this.activeModel.xmlmi.hideUnimportant =
        this.hide_unimportant.hasAttribute("checked");
    if (this.draw_as_tree == null)
      this.activeModel.xmlmi.drawTree = true;
    else
      this.activeModel.xmlmi.drawTree =
        this.draw_as_tree.hasAttribute("checked");
  }
}

function SelectRawDataTreeLayout(w)
{
  // Remove the existing active source, if any...
  if (w.rdfsource_active != null)
  {
    w.tree.database.RemoveDataSource(w.rdfsource_active);
    w.rdfsource_active = null;
  }

  // Figure out the new active source...
  // w.rdfsource_active 
  var hu = w.hide_unimportant.hasAttribute("checked");
  var dat = w.draw_as_tree.hasAttribute("checked");

  if (hu && dat)
    w.rdfsource_active = w.rdfsource_important_tree;
  else if (hu && !dat)
    w.rdfsource_active = w.rdfsource_important_flat;
  else if (!hu && dat)
    w.rdfsource_active = w.rdfsource_everything_tree;
  else
    w.rdfsource_active = w.rdfsource_everything_flat;

  w.tree.database.AddDataSource(w.rdfsource_active);
  w.tree.builder.rebuild();
}

function IncreaseTime(w)
{
  // Jump by 9 times the grid size...
  var jump = 9 * w.activeModel.gridx;
  var newval = parseFloat(w.timebox.value) + jump;
  // Work around a Mozilla bug...
  w.imageup.setAttribute("onclick",
                         "IncreaseTime(this.parentNode.parentNode." +
                         "parentNode.vwin);");
  if (newval > w.activeModel.stopx)
    newval = w.activeModel.stopx;
  if (newval < w.activeModel.startx)
    newval = w.activeModel.startx;

  w.timebox.value = DoubleToPresentationString(newval, 5);
  ChangeVisibleResultSet(w);
}

function DecreaseTime(w)
{
  // Jump by 9 times the grid size...
  var jump = -9 * w.activeModel.gridx;
  var newval = parseFloat(w.timebox.value) + jump;

  if (newval > w.activeModel.stopx)
    newval = w.activeModel.stopx;
  if (newval < w.activeModel.startx)
    newval = w.activeModel.startx;
  
  w.timebox.value = DoubleToPresentationString(newval, 5);
  ChangeVisibleResultSet(w);
  // Work around a Mozilla bug...
  w.imagedown.setAttribute("onclick",
                           "DecreaseTime(this.parentNode.parentNode." +
                           "parentNode.vwin);");
}

function RawDataChangeHideUnimportant(cb)
{
  SelectRawDataTreeLayout(cb.parentNode.parentNode.vwin);
}

function RawDataChangeTreeDraw(cb)
{
  SelectRawDataTreeLayout(cb.parentNode.parentNode.vwin);
}

function RawDataChangeImportant(cb)
{
  var w = cb.parentNode.parentNode.vwin;
  var tc = w.tree.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  var idx = GetRDFIndexFromTree(tc.window, tc);
  if (idx == -1)
    return;

  w.vlevs[idx].setImportant((cb.hasAttribute("checked")));
}

function ResultVariableSelected(vwin, tree)
{
  tree = tree.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  var idx = GetRDFIndexFromTree(vwin, tree);
  if (idx == -1 || vwin.vlevs[idx] == null)
  {
    vwin.important.setAttribute("disabled", "true");
    return;
  }

  vwin.important.removeAttribute("disabled");

  if (vwin.vlevs[idx].important)
    vwin.important.setAttribute("checked", "true");
  else
    vwin.important.removeAttribute("checked");
}

function GetVariableDescriptor(v, i)
{
  var vd = v[i].name;
  try
  {
    vd += "(" + v[i].units + ")";
  }
  catch (e)
  {
  }
  return vd;
}

function StreamPrint(strm, strng)
{
  strng += "\r\n";
  strm.write(strng, strng.length);
}

function SaveAsCSV(b)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var model = b.parentNode.parentNode.vwin.activeModel;
  if (model == null)
  {
    GetStatusBar().setAttribute("label", "Select a model first.");
    return;
  }
  const nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
    .createInstance(nsIFilePicker);
  fp.init(window, strb.getString("chooseSaveFilename"),
          nsIFilePicker.modeSave);
  fp.appendFilter("CSV", "*.csv");
  fp.appendFilter("TXT", "*.txt");
  fp.appendFilter(strb.getString("allFiles"), "*");
  if (fp.show() == nsIFilePicker.returnCancel)
    return;
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
    .createInstance(Components.interfaces.nsIFileOutputStream);
  var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
    .createInstance(Components.interfaces.nsIDOMSerializer);
  stream.init(
              fp.file,
              /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */
              42,
              /*0644*/420,
              0
             );
  
  // We now have to write to the stream...
  if (model.xmlmi.nivariables == 0)
  {
    stream.close();
    return;
  }
  var names = "";
  names += model.xmlmi.bvar.name + "(" + model.xmlmi.bvar.units + ")";
  var i;
  for (i = 0; i < model.xmlmi.nivariables; i++)
    names += "," + GetVariableDescriptor(model.xmlmi.ivariables, i);
  StreamPrint(stream, names);
  
  try
  {
    i = 0;
    var boundval = model.startx;
    for (i = 0; true; i++)
    {
      var values = "" + boundval;
      boundval += model.gridx;
      var j;
      for (j = 0; j < model.xmlmi.nivariables; j++)
        values += "," + model.results.getValue(j, i);
      StreamPrint(stream, values);
    }
  }
  catch (ex)
  {
  }
  stream.close();
  GetStatusBar().setAttribute("label", strb.getString("csvExportComplete"));
}

function RawResultsSaveContent(node)
{
  // This function is a placeholder for now, as integration results are not
  // saved, and so there is no need to restore raw result windows.
}

function RawResultsRestoreContent(node)
{
  // This function is a placeholder for now, as integration results are not
  // saved, and so there is no need to restore raw result windows.
}
