function SafeCORBA(x, irw)
{
  var laste;
  var i;
  for (i = 0; i < 3; i++)
  {
    try
    {
      irw.p.handlePendingMessages();
      return x();
    }
    catch (e)
    {
      laste = e;
    }
  }
  alert("Error repeated 3 times: " + laste);
  throw laste;
}

/*
 *  Sets up Zinc(including the Zinc feeder).
 *  Expected call-context: modelview.xul, ui thread
 *  @param irw
 *     The integration result watcher for the integrator being set up.
 */
function SetupZincForNewIntegrator(irw)
{
  SetupCmguiAxes(irw);
  SetupVariablesInCmgui(irw);

  createScene(irw, 0);
  irw.zincFeeder =
    irw.integrator.sendPointsToZinc(
                                    irw, irw.startx, irw.stopx, irw.gridx,
                                    irw.variables, irw.nvariables,
                                    irw.fields, irw.nvariables,
                                    irw.commandData,
                                    irw.timeSequencePackage,
                                    irw.mutableTimePoint,
                                    irw.dataNode
                                   );

  irw.zincFeeder.QueryInterface(Components.interfaces.mozIPointToZincLocal)
     .setPlugin(irw.p);
}

/*
 *  Sets up the context so that subsequent calls to CmguiCommand will not take
 *  effect until CommitCmguiCommands is called.
 *  Expected call-context: modelview.xul or integrationresult.xul, ui thread
 *  @param irw
 *    The integration result watcher.
 */
function BeginCmguiCommands(irw)
{
  irw.cmguiTransMode = true;
  irw.cmguiCommands = "";
}

/*
 *  Specifies a cmgui command to be performed.
 *  Expected call-context: modelview.xul or integrationresult.xul, ui thread
 *  @param irw
 *    The integration result watcher.
 *  @param cmd
 *    The command to execute
 */
function CmguiCommand(irw, cmd)
{
  var wrapped = "cmiss::cmiss(\"" + cmd + "\");\n";
  if (irw.cmguiTransMode)
    irw.cmguiCommands += wrapped;
  else
  {
    SafeCORBA(function(){irw.commandData.executeCommand(wrapped);}, irw);
  }
}

/*
 *  Executes the commands specified by any call to CmguiCommand since the call
 *  to BeginCmguiCommands. Until the next call to BeginCmguiCommand, any calls
 *  to CmguiCommand will execute immediately.
 *  Expected call-context: modelview.xul or integrationresult.xul, ui thread
 *  @param irw
 *    The integration result watcher.
 */
function CommitCmguiCommands(irw)
{
  SafeCORBA(function(){irw.commandData.executeCommand(irw.cmguiCommands)}, irw);
  irw.cmguiTransMode = false;
  irw.cmguiCommands = "";
}

/*
 *  Draws axes and sets up the fields for each variable.
 *  @param irw
 *    The integration result watcher.
 */
function SetupCmguiAxes(irw)
{
  var rootRegion = SafeCORBA(function(){return irw.commandData.rootRegion;}, irw);

  var visibleNode = SafeCORBA(function(){return rootRegion.createNode(1001);}, irw);

  /* Create a "coordinates" field */
  var coordinates = SafeCORBA(function(){
				return rootRegion.createStandardInterpolation(
				    "system:coordinates",
				    ["x", "y", "z"],
				    3 /* 3 components... */
				    );}, irw);

  /* The default creator with no derivatives or versions is suitable for this field */
  var fieldCreator = SafeCORBA(function(){return irw.commandData.createNodeFieldCreator(3);}, irw);

  /* All visible nodes have a coordinate defined... */
  SafeCORBA(function() {coordinates.defineOnNode(visibleNode, null, fieldCreator)}, irw);

  // This node is to contains all the data...
  var dataNodeA = SafeCORBA(function(){return visibleNode.createNode(1);}, irw);
  var dataNodeP = SafeCORBA(function(){return visibleNode.createNode(1);}, irw);
  irw.dataNode = dataNodeP;

  var npoints = parseInt((irw.stopx - irw.startx) / irw.gridx);
  if (irw.startx + npoints * irw.gridx == irw.stopx)
    npoints++;

  irw.timeSequencePackage =
    SafeCORBA(function(){return irw.commandData.createTimeSequencePackage();}, irw);
  var timeSeq = SafeCORBA(
		  function(){return irw.timeSequencePackage
		                       .getSimpleTimeSequence
		      (npoints, irw.startx, irw.gridx);}, irw);

  var mutableTimePoint =
    SafeCORBA(function(){
	        return irw.timeSequencePackage.getMatchingTimeSequence([irw.startx], 1);
	      }, irw);
  timepoints = null;

  /* Now we need to create a field for each variable... */
  irw.fields = new Array();
  var fieldCreatorPoint =
    SafeCORBA(function(){
		return irw.commandData.createNodeFieldCreator(1);
	      }, irw);
  var i;
  for (i = 0; i < irw.nvariables; i++)
  {
    var v = irw.variables[i];
    var fn = "field:v" + i;
    var fi = SafeCORBA(function(){
                 return rootRegion.createStandardInterpolation(
                   "field:v" + i, ["value"],
                   1 /* 1 component... */
                                                              );
	       }, irw);
    irw.fields[i] = fi;
    SafeCORBA(function(){fi.defineOnNode(dataNodeA, timeSeq, fieldCreatorPoint);}, irw);
    SafeCORBA(function(){fi.defineOnNode(dataNodeP, mutableTimePoint, fieldCreatorPoint);}, irw);
  }

  var minSignal = SafeCORBA(function(){return visibleNode.createNode(2);},
  		    irw);
  var maxSignal = SafeCORBA(function(){return visibleNode.createNode(3);},
			    irw);
  var minBound = dataNodeP;
  var maxBound = SafeCORBA(function(){return dataNodeP.createNode(5);},
			   irw);

  var nv =
    SafeCORBA(function(){ return coordinates.getNodalValues(
    "", 0, minSignal,
    Components.interfaces.CmguiIFunction.NODAL_VALUE,
    0,
    null
                            );
	      }, irw);

  var m = SafeCORBA(function(){return irw.commandData.createMatrix(3, [0, -1000, 0], 1*3)},
		    irw);
  SafeCORBA(function(){nv.setValue(m);}, irw);

  var nv =
    SafeCORBA(function(){
		return coordinates.getNodalValues(
				 "", 0, maxSignal,
				 Components.interfaces.CmguiIFunction.NODAL_VALUE,
				 0,
				 null
				)
		  }, irw);
  var m =
    SafeCORBA(function(){
		return irw.commandData.createMatrix(3, [0, 1000, 0], 1*3)},
	      irw);
  SafeCORBA(function(){
		nv.setValue(m);},
	    irw);

  var nv =
    SafeCORBA(function(){ return coordinates.getNodalValues(
    "", 0, minBound,
    Components.interfaces.CmguiIFunction.NODAL_VALUE,
    0,
    null
                            );
                        }, irw);

  var m = SafeCORBA(function(){return irw.commandData.createMatrix(3, [0, 0, 0], 1*3)},
		    irw);
  SafeCORBA(function(){nv.setValue(m);}, irw);

  var x = SafeCORBA(function(){return coordinates.getNodalValues(
    "", 0, maxBound,
    Components.interfaces.CmguiIFunction.NODAL_VALUE,
    0,
    null
    );}, irw);
  var m = SafeCORBA(function()
		    {
		      return irw.commandData.createMatrix(3, [2000, 0, 0], 1*3);
		    }, irw);
  SafeCORBA(function(){x.setValue(m);}, irw);

  /* Make an element from these two nodes */
  var signalAxis = SafeCORBA(function(){return rootRegion.createElementWithLineShape(3, 1);},
                             irw);
  var boundAxis = SafeCORBA(function(){
                    return rootRegion.createElementWithLineShape(5, 1);
                            }, irw);


  SafeCORBA(function(){
    coordinates.defineTensorProductBasisOnElement(
      signalAxis, 1,
      Components.interfaces.CmguiIFunction.BASIS_LINEAR_LAGRANGE
                                                 );
            }, irw);

  SafeCORBA(function(){
    coordinates.defineTensorProductBasisOnElement(
      boundAxis, 1,
      Components.interfaces.CmguiIFunction.BASIS_LINEAR_LAGRANGE
                                                 );
            }, irw);

  SafeCORBA(function(){signalAxis.setNode(0, minSignal);}, irw);
  SafeCORBA(function(){signalAxis.setNode(1, maxSignal);}, irw);
  SafeCORBA(function(){boundAxis.setNode(0, minBound);}, irw);
  SafeCORBA(function(){boundAxis.setNode(1, maxBound);}, irw);

  SafeCORBA(function(){rootRegion.beginChange();}, irw);
  SafeCORBA(function(){rootRegion.mergeNode(minSignal);}, irw);
  SafeCORBA(function(){rootRegion.mergeNode(maxSignal);}, irw);
  SafeCORBA(function(){rootRegion.mergeNode(minBound);}, irw);
  SafeCORBA(function(){rootRegion.mergeNode(maxBound);}, irw);
  SafeCORBA(function(){rootRegion.mergeNode(dataNodeA);}, irw);
  SafeCORBA(function(){rootRegion.mergeNode(dataNodeP);}, irw);
  SafeCORBA(function(){rootRegion.mergeElement(signalAxis);}, irw);
  SafeCORBA(function(){rootRegion.mergeElement(boundAxis);}, irw);
  SafeCORBA(function(){rootRegion.endChange();}, irw);

  var blackProperties = "diffuse 0 0 0 specular 0 0 0 shininess 0";
  BeginCmguiCommands(irw);
  var scale = irw.stopx;
  CmguiCommand(irw, "gfx modify scene default manual;");
  CmguiCommand(irw,
               "gfx define field timebase scale field xi.1 scale_factor " +
               scale + ";");
  CmguiCommand(irw, "gfx create material system:black " + blackProperties);
  CommitCmguiCommands(irw);
  irw.mutableTimePoint = mutableTimePoint;
}

function createScene(irw, i)
{
  var generalProperties = "circle_discretization 6 " +
                          "default_coordinate system:coordinates " +
                          "element_discretization \\\"50*50*50\\\" " +
                          "native_discretization none scene scene" + i;
  var axesLineProperties = "select_on " +
                           "material system:black " +
                           "selected_material system:black " +
                           "line_width 1";

  BeginCmguiCommands(irw);
  CmguiCommand(irw, "gfx create scene scene" + i + " manual;");
  CmguiCommand(irw, "gfx modify scene scene" + i + " add_light default;");
  CmguiCommand(irw,
               "gfx create egroup scene" + i + ":axes;");
  CmguiCommand(irw, "gfx draw group scene" + i + ":axes scene scene" + i);
  CmguiCommand(irw,
               "gfx modify egroup scene" + i + ":axes add 3;");
  CmguiCommand(irw, "gfx modify egroup scene" + i + ":axes add 5;");
  CmguiCommand(irw,
               "gfx modify g_element scene" + i + ":axes general clear " +
               generalProperties + ";");
  CmguiCommand(irw,
               "gfx modify g_element scene" + i + ":axes lines " +
               axesLineProperties + " scene scene" + i + ";");
  CommitCmguiCommands(irw);

  for (j = 0; j < irw.nvariables; j++)
  {
    BeginCmguiCommands(irw);
    CmguiCommand(irw,
                 "gfx create egroup scene" + i + ":trace:v" + j + ";");
    CmguiCommand(irw, "gfx draw group scene" + i + ":trace:v" + j +
                 " scene scene" + i);
    CmguiCommand(irw, "gfx modify egroup scene" + i + ":trace:v" + j +
                 " add 5;");
    CmguiCommand(irw,
                 "gfx modify g_element scene" + i + ":trace:v" + j + " general clear " +
                 generalProperties + ";");
    CmguiCommand(irw,
                 "gfx modify g_element scene" + i + ":trace:v" + j + " lines " +
                 "coordinate plot:v" + j +
                 " select_on" +
                 " material material:v" + j +
                 " line_width 1 scene scene" + i + ";");
    CommitCmguiCommands(irw);
  }
}

function destroyScene(irw, i)
{
  CmguiCommand(irw, "gfx destroy scene scene" + i);
}

/*
 * Sets up all the variables in cmgui.
 * 
 */
function SetupVariablesInCmgui(irw)
{
  var generalProperties = "circle_discretization 6 " +
                          "default_coordinate system:coordinates " +
                          "element_discretization \\\"50*50*50\\\" " +
                          "native_discretization none";
  var i;
  for (i = 0; i < irw.nvariables; i++)
  {
    // TODO: Lookup these colours for the graph.
    var col = irw.colour[i];
    var red = col & 0xFF, green = (col >> 8) & 0xFF, blue = (col >> 16) & 0xFF;
    red = red / 255.0;
    green = green / 255.0;
    blue = blue / 255.0;

    BeginCmguiCommands(irw);
    CmguiCommand(irw,
                 "gfx create material material:v" + i +
                 " diffuse " + red + " " + green + " " + blue +
                 " specular 0.5 0.5 0.5 shininess 0;");
    CmguiCommand(irw,
               "gfx define field nodal:v" + i + " nodal_lookup field " +
               "field:v" + i + " node 1;");
    CmguiCommand(irw,
               "gfx define field offset:v" + i +
               " coordinate_system rectangular_cartesian " +
               "composite 0 nodal:v" + i + " 0;");
    ComputeCmguiScale(irw, i, 1000);
    CmguiCommand(irw,
                 "gfx define field plot:v" + i +
                 " time_lookup field coordinate:v" + i +
                 " time timebase;");
    CommitCmguiCommands(irw);
  }
}

/*
 *  Changes the scale of a variable.
 *  @param irw
 *    The integration result watcher.
 *  @param i
 *    The index of the variable being changed.
 *  @param maxAbs
 *    The new maximum absolute value.
 */
function ComputeCmguiScale(irw, i, maxAbs)
{
  if (maxAbs == 0)
    maxAbs = 1000;
  var factor = 1000.0 / maxAbs;

  CmguiCommand(irw,
               "gfx define field coordinate:v" + i +
               " coordinate_system rectangular_cartesian " +
               "add fields offset:v" + i + " system:coordinates " +
               "scale_factors " + factor + " 1;");
}

function windowReadyCallback(w)
{
  this.win = w;
  return this;
}

windowReadyCallback.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.CmguiISceneViewerResultMonitor) ||
        aIID.equals(Components.interfaces.nsISupports)
       )
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
  },

  sceneViewerReady: function(sceneViewer)
  {
    var plug = this.tab.embed;
    sceneViewer.setWindowSize(plug.width, plug.height);

    this.tab.sceneViewer = sceneViewer;

    if (this.win.waiting_sv_ready)
    {
      this.irw.activateIntegration();
      this.win.zincFeeder = this.irw.zincFeeder;
      this.irw.zincFeeder = null;
    }

    var tid = this.tab.id;
    SafeCORBA(function(){
      sceneViewer.setSceneByName("scene" + tid);
              }, this.irw);

    SafeCORBA(function(){
      sceneViewer.setBackgroundColourRGB(1.0, 1.0, 1.0);
              }, this.irw);
    var svi = Components.interfaces.CmguiISceneViewer;

    SafeCORBA(function(){
      sceneViewer.viewAll();
              }, this.irw);

    // Register to learn when the window is resized...
    this.tab.embed.resizeMonitor =
      ProxyToMain(new NotifyingResizeMonitor(sceneViewer, this.irw),
                  Components.interfaces.mozIPluginResizeMonitor);

    SafeCORBA(function(){
      sceneViewer.viewportMode = svi.VIEWPORT_DISTORTING_RELATIVE;
              }, this.irw);
    SafeCORBA(function(){
      sceneViewer.tumbleRate = 0;
              }, this.irw);

    this.win.waiting_sv_ready = 0;

    this.win = null;
    this.irw = null;
  },

  sceneViewerFailed: function()
  {
  }
};

function NotifyingResizeMonitor(sv, irw)
{
  this.sceneViewer = sv;
  this.irw = irw;
  return this;
}

NotifyingResizeMonitor.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.mozIPluginResizeMonitor) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  onResize: function(width, height)
  {
    this.irw.p.handlePendingMessages();
    this.sceneViewer.setWindowSize(width, height);
  }
};

function SetupEmbed(tab, irw)
{
  var pluginEl = tab.embed;
  tab.pluginEl = pluginEl;
  var windowId = pluginEl.window;

  var wrc = new windowReadyCallback(window);
  
  wrc.irw = irw;
  wrc.tab = tab;

  var wrcmp = null;
  do
  {
    wrcmp = ProxyToMain(wrc, Components.interfaces
                                       .CmguiISceneViewerResultMonitor);
  }
  while (wrcmp == null);

  irw.p.handlePendingMessages();
  irw.commandData.createSceneViewer
  (
   windowId,
   wrcmp
  );
}

function CmguiSetVariableVisibility(j, i, vis)
{
  CmguiCommand(window.arguments[0],
               "gfx modify g_element scene" + j + ":trace:v" + i + " lines " +
               "coordinate plot:v" + i +
               " select_on" +
               " material material:v" + i +
               " line_width 1 " + (vis ? "visible" : "invisible") +
               " scene scene" + j + ";");
}

function CmguiSetVariableColour(idx, red, green, blue)
{
  red = red / 255.0;
  green = green / 255.0;
  blue = blue / 255.0;
  CmguiCommand(window.arguments[0],
               "gfx modify material material:v" + idx +
               " diffuse " + red + " " + green + " " + blue +
               " specular 0.5 0.5 0.5 shininess 0;");
}
