#ifdef XP_WIN
#include <windows.h>
#endif
#include "mozGeneralPlugin.h"

NS_IMPL_ISUPPORTS2(mozGeneralPlugin, mozIGeneralPlugin, nsIClassInfo)

nsPluginInstanceBase*
NS_NewPluginInstance(nsPluginCreateData* aCreateDataStruct)
{
  mozGeneralPlugin* mgpi = new mozGeneralPlugin();
  NS_ADDREF(mgpi);
  return mgpi;
}

void
NS_DestroyPluginInstance(nsPluginInstanceBase* aPlugin)
{
  NS_STATIC_CAST(mozGeneralPlugin*, aPlugin)->shut();
  mozGeneralPlugin* gp = NS_STATIC_CAST(mozGeneralPlugin*, aPlugin);
  NS_RELEASE(gp);
}

NPError
NS_PluginInitialize()
{
  return NPERR_NO_ERROR;
}

void
NS_PluginShutdown()
{
}

char*
NPP_GetMIMEDescription()
{
  return "application/x-general-plugin::Mozilla General Plugin";
}


#ifdef XP_UNIX
NPError
NS_PluginGetValue(NPPVariable aVariable, void *aValue)
{
  if (aVariable == NPPVpluginNameString)
  {
    *((char **)aValue) = "Mozilla General Plugin";
    return NPERR_NO_ERROR;
  }
  else if (aVariable == NPPVpluginDescriptionString)
  {
    *((char**)aValue) = "A plugin that lets applications get a window handle.";
    return NPERR_NO_ERROR;
  }
  return NPERR_INVALID_PARAM;
}
#endif

mozGeneralPlugin::mozGeneralPlugin()
  : inited(FALSE)
{
}

NPBool
mozGeneralPlugin::init(NPWindow* aWindow)
{
  inited = TRUE;
  SetWindow(aWindow);
  return TRUE;
}

void
mozGeneralPlugin::shut()
{
  inited = FALSE;
  resizeMonitor = nsnull;
}

NPBool
mozGeneralPlugin::isInitialized()
{
  return inited;
}

NPError
mozGeneralPlugin::GetValue(NPPVariable variable, void *value)
{
  static nsIID scriptableIID = MOZIGENERALPLUGIN_IID;
  nsIID* ptr;

  switch (variable)
  {
  case NPPVpluginNameString:
    *((char **)value) = "Mozilla General Plugin";
    return NPERR_NO_ERROR;
  case NPPVpluginDescriptionString:
    *((char **)value) = "Mozilla General Plugin";
    return NPERR_NO_ERROR;
  case NPPVpluginScriptableInstance:
    NS_ADDREF(this);
    *((nsISupports**)value) = NS_STATIC_CAST(mozIGeneralPlugin*, this);
    return NPERR_NO_ERROR;
  case NPPVpluginScriptableIID:
    ptr = (nsIID*)NPN_MemAlloc(sizeof(nsIID));
    if (ptr)
    {
      *ptr = scriptableIID;
      *(nsIID **)value = ptr;
    }
    else
      return NPERR_OUT_OF_MEMORY_ERROR;
    return NPERR_NO_ERROR;
  case NPPVpluginNeedsXEmbed:
    *(int*)value = 1;
    return NPERR_NO_ERROR;
  }
  return NPERR_INVALID_PARAM;
}

NPError
mozGeneralPlugin::SetWindow(NPWindow* aWindow)
{
  window = (PRUint32)(aWindow->window);
  width = aWindow->width;
  height = aWindow->height;

  if (resizeMonitor != nsnull)
  {
    resizeMonitor->OnResize(width, height);
  }
  return NPERR_NO_ERROR;
}

NS_IMETHODIMP
mozGeneralPlugin::GetWindow(PRUint32* owindow)
{
  *owindow = window;
  return NS_OK;
}

NS_IMETHODIMP
mozGeneralPlugin::GetWidth(PRUint32* owidth)
{
  *owidth = width;
  return NS_OK;
}

NS_IMETHODIMP
mozGeneralPlugin::GetHeight(PRUint32* oheight)
{
  *oheight = height;
  return NS_OK;
}


NS_IMETHODIMP
mozGeneralPlugin::GetFlags(PRUint32 *aFlags)
{
  *aFlags = nsIClassInfo::PLUGIN_OBJECT | nsIClassInfo::DOM_OBJECT;
  return NS_OK;
}

NS_IMETHODIMP
mozGeneralPlugin::GetImplementationLanguage(PRUint32 *aImplementationLanguage)
{
  *aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
  return NS_OK;
}

NS_IMETHODIMP
mozGeneralPlugin::GetInterfaces(PRUint32 *count, nsIID * **array)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozGeneralPlugin::GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozGeneralPlugin::GetContractID(char * *aContractID)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozGeneralPlugin::GetClassDescription(char * *aClassDescription)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozGeneralPlugin::GetClassID(nsCID * *aClassID)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozGeneralPlugin::GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
{
  return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
mozGeneralPlugin::GetResizeMonitor(mozIPluginResizeMonitor** rm)
{
  *rm = resizeMonitor;
  if (*rm != nsnull)
    NS_ADDREF(*rm);
  return NS_OK;
}

NS_IMETHODIMP
mozGeneralPlugin::SetResizeMonitor(mozIPluginResizeMonitor* rm)
{
  resizeMonitor = rm;
  return NS_OK;
}

NS_IMETHODIMP
mozGeneralPlugin::HandlePendingMessages()
{
#ifdef XP_WIN
  MSG msg;

  while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE | PM_NOYIELD))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
#endif
  return NS_OK;
}
