#!/usr/bin/python
# Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
# http://xpcom2corba.sunsite.dk/

# XPCOM-CORBA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

TMPL_proxy_h_file_header = """\
//	-*- C++ -*-
// generated automagically by XPCOM-CORBA
#ifndef @guardname@
#define @guardname@

// include CORBA stub
#include "@CORBAprefix@@basename@@CORBAsuffix@"

// include XPCOM interface declaration
#include "@XPCOMprefix@@basename@@XPCOMsuffix@"

#include <nsMemory.h>
#include <nsString.h>

#include <xpcom_object.h>

#include <xpcom2corba.h>
#include <corba2xpcom.h>
"""

TMPL_proxy_h_file_footer = '#endif'


TMPL_proxy_h_sequence = """\
inline void xpcom2corba(@xpcom_childname@ *xpcom_obj, PRUint32 length, @corba_name@ &corba_obj)
{
  corba_obj.length(length);

  for (PRUint32 i = 0; i < length; i++)
  {
    ::xpcom2corba(xpcom_obj[i], corba_obj[i]);
  }
}
"""

TMPL_proxy_h_sequence_string = """\
inline void xpcom2corba(const @xpcom_childname@ const *xpcom_obj, PRUint32 length, @corba_name@ &corba_obj)
{
  corba_obj.length(length);

  for (PRUint32 i = 0; i < length; i++)
  {
    ::xpcom2corba(xpcom_obj[i], corba_obj[i].out());
  }
}
"""

TMPL_proxy_h_sequence_objref = """\
inline void xpcom2corba(@xpcom_childname@ **xpcom_obj, PRUint32 length, @corba_name@ &corba_obj)
{
  corba_obj.length(length);

  for (PRUint32 i = 0; i < length; i++)
  {
    @corba_childname@* type_ptr = (@corba_childname@*)NULL;
    @corba_childname@_ptr ptr;
    ::xpcom2corba(xpcom_obj[i], type_ptr, &ptr);
    corba_obj[i] = ptr;
  }
}
"""


TMPL_proxy_h_struct_x2c_header = """\
inline void xpcom2corba(@xpcom_name@ *xpcom_obj, @corba_name@ &corba_obj)
{"""

TMPL_proxy_h_struct_x2c_footer = """\
}
"""


TMPL_proxy_h_intf_header_inherit = """
class @intfname@__XPCOM_Proxy
  : public @intfname@,
    public @supername@
{
 public:
  NS_DECL_ISUPPORTS_INHERITED
  NS_DECL_@uppername@

  explicit @intfname@__XPCOM_Proxy(@fqname@_ptr corba_obj);
  @intfname@__XPCOM_Proxy(@fqname@_ptr corba_obj, PRInt32 *ref_count);
  virtual ~@intfname@__XPCOM_Proxy();
"""

TMPL_proxy_h_intf_header_noinherit = """
class @intfname@__XPCOM_Proxy
  : public @intfname@,
    public @supername@
{
 public:
  NS_DECL_ISUPPORTS
  NS_DECL_@uppername@

  explicit @intfname@__XPCOM_Proxy(@fqname@_ptr corba_obj);
  @intfname@__XPCOM_Proxy(@fqname@_ptr corba_obj, PRInt32 *ref_count);
  virtual ~@intfname@__XPCOM_Proxy();
"""

TMPL_proxy_h_intf_footer = """
 private:
  @fqname@_var corba_obj_;
};
"""

TMPL_proxy_h_operation = """\
// NS_IMETHODIMP @capname@(@paramspec@);\
"""


TMPL_proxy_c_file_header = """\
//	-*- C++ -*-
// generated automagically by XPCOM-CORBA
#include "@basename@_xpcom.h"
#include "@basename@_corba.h"

#include <xpcom_proxy.h>
#include <corba_poa.h>
"""

TMPL_proxy_c_intf = """
NS_IMPL_ISUPPORTS_INHERITED1(@intfname@__XPCOM_Proxy, @supername@, @intfname@)

@intfname@__XPCOM_Proxy::@intfname@__XPCOM_Proxy(@fqname@_ptr corba_obj)
  : @supername@(corba_obj),
    corba_obj_(@fqname@::_duplicate(corba_obj))
{
}

@intfname@__XPCOM_Proxy::@intfname@__XPCOM_Proxy(@fqname@_ptr corba_obj, PRInt32 *ref_count)
  : @supername@(corba_obj, ref_count),
    corba_obj_(@fqname@::_duplicate(corba_obj))
{
}

@intfname@__XPCOM_Proxy::~@intfname@__XPCOM_Proxy()
{
}


class @intfname@__XPCOM_Factory
  : public XPCOM_Proxy_Factory
{
 public:
  explicit @intfname@__XPCOM_Factory(const nsID &iid)
    : XPCOM_Proxy_Factory(iid, this)
  { }

  virtual ~@intfname@__XPCOM_Factory()
  { }


  virtual CORBA_IObject *GetProxy(CORBA::Object_ptr obj,
				  PRInt32 *ref_count) const;
};

CORBA_IObject *
@intfname@__XPCOM_Factory::GetProxy(CORBA::Object_ptr obj, PRInt32 *ref_count) const
{
  @fqname@_ptr narrowed_obj = @fqname@::_narrow(obj);
  if (narrowed_obj)
  {
    return static_cast<@intfname@ *>(new @intfname@__XPCOM_Proxy(narrowed_obj, ref_count));
  }
  else
  {
    return NULL;
  }
}

const @intfname@__XPCOM_Factory
  the__@intfname@__XPCOM_Factory(@intfname@::GetIID());
"""

TMPL_proxy_c_operation_header = """\
NS_IMETHODIMP @intfname@__XPCOM_Proxy::@capname@(@paramspec@)
{
  try
  {
    nsresult _result = NS_OK;
"""

TMPL_proxy_c_operation_stringout = """\
@stringvar@ = (char *) nsMemory::Clone(@stringvar@, sizeof (char) * (strlen(@stringvar@) + 1));
if (@stringvar@ == 0)
{
  _result = NS_ERROR_OUT_OF_MEMORY;
}
"""

TMPL_proxy_c_operation_footer = """
    return _result;
  }
  catch (const CORBA::UserException &user_ex)
  {
    // TODO: return exception data
    return NS_ERROR_UNEXPECTED;
  }
  catch (const CORBA::SystemException &sys_ex)
  {
    // TODO: return exception data
    return NS_ERROR_UNEXPECTED;
  }
}
"""
