/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */

#include "xpcom_object.h"
#include "xpcom_proxy.h"

#include <ICORBAWrapper.h>


class CORBAWrapper_Impl
  : public ICORBAWrapper
{
 public:
  NS_DECL_ISUPPORTS
  NS_DECL_ICORBAWRAPPER


  CORBAWrapper_Impl(CORBA::Object_ptr corba_obj)
    : corba_obj_(corba_obj)
  { }

  virtual ~CORBAWrapper_Impl();


 private:
  CORBA::Object_var corba_obj_;
};


NS_IMPL_THREADSAFE_ISUPPORTS1(CORBAWrapper_Impl, ICORBAWrapper)

CORBAWrapper_Impl::~CORBAWrapper_Impl()
{
}

NS_IMETHODIMP CORBAWrapper_Impl::GetCorbaObj(CORBA::Object_ptr *aCorbaObj)
{
  *aCorbaObj = CORBA::Object::_duplicate(corba_obj_);

  return NS_OK;
}


NS_IMPL_THREADSAFE_ADDREF(CORBA_IObject__XPCOM_Proxy)
NS_IMPL_THREADSAFE_RELEASE(CORBA_IObject__XPCOM_Proxy)
NS_IMPL_QUERY_HEAD(CORBA_IObject__XPCOM_Proxy)
  if (aIID.Equals(NS_GET_IID(CORBA_IObject)))
  {
    foundInterface = NS_STATIC_CAST(nsISupports *, this);
  }
  else if (aIID.Equals(NS_GET_IID(ICORBAWrapper)))
  {
    foundInterface = new CORBAWrapper_Impl(CORBA::Object::_duplicate(corba_obj_));
  }
  else
  {
    foundInterface = XPCOM_Proxy_Factory::GetProxy(aIID, corba_obj_, NULL);
  }

  nsresult status;
  if (foundInterface)
  {
    NS_ADDREF(foundInterface);
    status = NS_OK;
  }
  else
  {
    status = NS_NOINTERFACE;
  }
  *aInstancePtr = foundInterface;
  return status;
}

CORBA_IObject__XPCOM_Proxy::CORBA_IObject__XPCOM_Proxy(CORBA::Object_ptr corba_obj)
  : corba_obj_(CORBA::Object::_duplicate(corba_obj))
{
  NS_INIT_ISUPPORTS();
}

CORBA_IObject__XPCOM_Proxy::CORBA_IObject__XPCOM_Proxy(CORBA::Object_ptr corba_obj, PRInt32 *ref_count)
  : corba_obj_(CORBA::Object::_duplicate(corba_obj))
{
  NS_INIT_ISUPPORTS();
}

CORBA_IObject__XPCOM_Proxy::~CORBA_IObject__XPCOM_Proxy()
{
}


NS_IMPL_THREADSAFE_ADDREF(XPCOM_IObject__XPCOM_Proxy)
NS_IMPL_THREADSAFE_RELEASE(XPCOM_IObject__XPCOM_Proxy)
NS_IMPL_QUERY_HEAD(XPCOM_IObject__XPCOM_Proxy)
  if (aIID.Equals(NS_GET_IID(CORBA_IObject)))
  {
    foundInterface = NS_STATIC_CAST(nsISupports *, this);
  }
  else if (aIID.Equals(NS_GET_IID(ICORBAWrapper)))
  {
    foundInterface = new CORBAWrapper_Impl(CORBA::Object::_duplicate(corba_obj_));
  }
  else
  {
    foundInterface = XPCOM_Proxy_Factory::GetProxy(aIID, corba_obj_, ref_count_);
  }

  nsresult status;
  if (foundInterface)
  {
    NS_ADDREF(foundInterface);
    status = NS_OK;
  }
  else
  {
    status = NS_NOINTERFACE;
  }
  *aInstancePtr = foundInterface;
  return status;
}

XPCOM_IObject__XPCOM_Proxy::XPCOM_IObject__XPCOM_Proxy(XPCOM::IObject_ptr corba_obj)
  : corba_obj_(XPCOM::IObject::_duplicate(corba_obj)),
    ref_count_(new PRInt32(1))
{
  *ref_count_ = 1;
  NS_INIT_ISUPPORTS();
}

XPCOM_IObject__XPCOM_Proxy::XPCOM_IObject__XPCOM_Proxy(XPCOM::IObject_ptr corba_obj, PRInt32 *ref_count)
  : corba_obj_(XPCOM::IObject::_duplicate(corba_obj)),
    ref_count_(ref_count)
{
  ++(*ref_count_);
  NS_INIT_ISUPPORTS();
}

XPCOM_IObject__XPCOM_Proxy::~XPCOM_IObject__XPCOM_Proxy()
{
  if (--(*ref_count_) <= 0)
  {
    delete ref_count_;

    try
    {
    corba_obj_->release_ref();
    }
    catch (...) {}
  }
}
